import {
  EdgePathLabelModel,
  ExteriorLabelModel,
  ExteriorLabelModelPosition,
  GraphComponent,
  IEdge,
  ILabel,
  ILabelModelParameter, IModelItem,
  INode,
  IOrientedRectangle,
  MutableRectangle,
  Point,
  SimpleLabel,
  SimpleNode,
  Size
} from 'yfiles'
import * as React from 'react'
import { EdgeSetupProps, MapConnection, MapObject } from './MapsCompose'
import { NodeStyleSettings } from './styles/NodeStyleSettings'
import { EdgeStyleSettings } from './styles/EdgeStyleSettings'

interface ContextProps {
    onUpdateNodeSetup: Function,
    onUpdateEdgeSetup: Function,
    onUpdateNodeLevel: Function,
    edgeSetup: EdgeSetupProps
}

export default class ContextualToolbar {
    private graphComponent: GraphComponent;
    private readonly container: HTMLElement;
    public selectedItems: IModelItem[];
    private readonly nodeLabelModelParameter: ILabelModelParameter;
    private readonly edgeLabelModelParameter: ILabelModelParameter;
    private dirty: boolean;
    private containsEdges: boolean;
    private containsShapeNodes: boolean;
    private containsLabels: boolean;
    private props: ContextProps;

    constructor (graphComponent: GraphComponent, container: HTMLElement, props: ContextProps) {
      this.graphComponent = graphComponent
      this.container = container
      this.props = props

      // initialize a label model parameter that is used to position the node pop-up
      const nodeLabelModel = new ExteriorLabelModel({ insets: 10 })
      this.nodeLabelModelParameter = nodeLabelModel.createParameter(ExteriorLabelModelPosition.NORTH)

      // initialize a label model parameter that is used to position the edge pop-up
      const edgeLabelModel = new EdgePathLabelModel({ autoRotation: false })
      this.edgeLabelModelParameter = edgeLabelModel.createDefaultParameter()

      this.selectedItems = []
      this.dirty = false

      this.registerUpdateListeners()
      this.registerClickListeners()
    }

    setSelectedItems (array: IModelItem[]) {
      if (array === null) {
        // eslint-disable-next-line no-throw-literal
        throw "SelectedItems can't be null. To hide the toolbar, set an empty array."
      }
      this.selectedItems = array
      this.containsEdges = this.getSelectedEdges().length > 0

      this.containsShapeNodes = this.getSelectedNodes().filter(item => {
        const data = item.tag as MapObject
        return parseInt(data.shapetype) > 0 ? item : null
      }).length > 0
      this.containsLabels = this.getSelectedLabels().length > 0
      if (array.length > 0) {
        this.show()
      } else {
        this.hide()
      }
    }

    applyNodeSetup (prop: string, value: string) {
      const labels = this.getSelectedLabels()
      labels.forEach(label => {
        const node = label.owner as INode
        const data = node.tag as MapObject
        const nodeSetup = NodeStyleSettings.getNodeSetupByMapObject(data)
        switch (prop) {
          case 'color': nodeSetup.color = value; break
          case 'background_color': nodeSetup.background_color = value; break
          case 'is_bold': nodeSetup.is_bold = value == 'true'; break
          case 'is_italic': nodeSetup.is_italic = value == 'true'; break
          case 'shape': nodeSetup.shape_type = parseInt(value); break
        }
        this.props.onUpdateNodeSetup(nodeSetup, true)
      })
    }

    changeFontSize (doIncrease: boolean) {
      const labels = this.getSelectedLabels()
      labels.forEach(label => {
        const node = label.owner as INode
        const data = node.tag as MapObject
        const nodeSetup = NodeStyleSettings.getNodeSetupByMapObject(data)
        nodeSetup.size = doIncrease ? nodeSetup.size + 2 : nodeSetup.size - 2
        this.props.onUpdateNodeSetup(nodeSetup, true)
      })
    }

    changeLevel (doIncrease: boolean) {
      const nodes = this.getSelectedNodes()
      nodes.forEach(node => {
        this.props.onUpdateNodeLevel(node, doIncrease)
      })
    }

    applyEdgeSetup (prop: string, value: string) {
      const edges = this.getSelectedEdges()
      edges.forEach(edge => {
        const data = edge.tag as MapConnection
        const edgeSetup = EdgeStyleSettings.getEdgeSetupByMapConnection(data)
        switch (prop) {
          case 'source_arrow': edgeSetup.source_arrow_type = parseInt(value); break
          case 'target_arrow': edgeSetup.target_arrow_type = parseInt(value); break
          case 'line_color': edgeSetup.line_color = value; break
          case 'stroke': edgeSetup.stroke = parseInt(value); break
          case 'style': edgeSetup.style = parseInt(value); break
        }
        this.props.onUpdateEdgeSetup(edgeSetup, true)
      })
    }

    showPickerContainer (e: React.MouseEvent<HTMLInputElement>) {
      const toggleButton = e.target as HTMLInputElement
      const pickerContainer = document.getElementById(toggleButton.getAttribute('data-container-id'))
      const show = toggleButton.checked

      if (!show) {
        this.hideAllPickerContainer()
        return
      }

      // hide all picker containers except for the one that should be toggled
      this.hideAllPickerContainer(toggleButton, pickerContainer)

      // position the container above/below the toggle button
      pickerContainer.style.display = 'block'
      const labelElement = document.querySelector(`label[for="${toggleButton.id}"]`)
      const labelBoundingRect = labelElement.getBoundingClientRect()
      const toolbarClientRect = this.container.getBoundingClientRect()
      const pickerClientRect = pickerContainer.getBoundingClientRect()
      pickerContainer.style.left = `${labelBoundingRect.left +
        labelBoundingRect.width / 2 -
        pickerContainer.clientWidth / 2 -
        toolbarClientRect.left}px`
      const gcAnchor = this.graphComponent.toPageFromView(new Point(0, 0))
      if (toolbarClientRect.top - gcAnchor.y < pickerClientRect.height + 20) {
        pickerContainer.style.top = '55px'
        this.addClass(pickerContainer, 'bottom')
      } else {
        pickerContainer.style.top = `-${pickerClientRect.height + 12}px`
        this.removeClass(pickerContainer, 'bottom')
      }

      // timeout the fading animation to make sure that the element is visible
      setTimeout(() => {
        pickerContainer.style.opacity = '1'
      }, 0)
    }

    hideAllPickerContainer (exceptToggleButton?: HTMLInputElement, exceptContainer?: HTMLElement) {
      const toggleButtons = document.querySelectorAll('input[data-container-id]')
      for (let i = 0; i < toggleButtons.length; i++) {
        const btn = toggleButtons[i] as HTMLInputElement
        if (btn !== exceptToggleButton) {
          btn.checked = false
        }
      }

      const pickerContainers = document.querySelectorAll('.picker-container')
      for (let i = 0; i < pickerContainers.length; i++) {
        const container = pickerContainers[i] as HTMLElement
        if (container.style.opacity !== '0' && container !== exceptContainer) {
          container.style.opacity = '0'
          setTimeout(() => {
            container.style.display = 'none'
          }, 300)
        }
      }
    }

    getSelectedItems () {
      return this.selectedItems
    }

    getSelectedEdges () {
      return this.selectedItems.filter(item => IEdge.isInstance(item))
    }

    getSelectedNodes () {
      return this.selectedItems.filter(item => INode.isInstance(item))
    }

    getSelectedLabels () {
      const labels: ILabel[] = []
      this.selectedItems.forEach(item => {
        if (ILabel.isInstance(item)) {
          labels.push(item)
        } else if (INode.isInstance(item) || IEdge.isInstance(item)) {
          item.labels.forEach(label => {
            labels.push(label)
          })
        }
      })
      return labels
    }

    updateItemUI () {
      const nodeUI = document.getElementById('node-ui')
      const labelUI = document.getElementById('label-ui')
      const edgeUI = document.getElementById('edge-ui')
      const levelUI = document.getElementById('level-ui')
      if (this.containsShapeNodes) {
        this.addClass(this.container, 'node-ui-visible')
        nodeUI.style.display = 'inline-block'
      } else {
        this.removeClass(this.container, 'node-ui-visible')
        nodeUI.style.display = 'none'
      }
      if (this.containsLabels) {
        this.addClass(this.container, 'label-ui-visible')
        labelUI.style.display = 'inline-block'
      } else {
        this.removeClass(this.container, 'label-ui-visible')
        labelUI.style.display = 'none'
      }
      if (this.containsEdges) {
        this.addClass(this.container, 'edge-ui-visible')
        edgeUI.style.display = 'inline-block'

        this.removeClass(this.container, 'level-ui-visible')
        levelUI.style.display = 'none'
      } else {
        this.removeClass(this.container, 'edge-ui-visible')
        edgeUI.style.display = 'none'

        this.addClass(this.container, 'level-ui-visible')
        levelUI.style.display = 'inline-block'
      }
    }

    updateLabelControlState () {
      const labels = this.getSelectedLabels()
      if (labels.length > 0) {
        const firstLabel = labels[0]
        const node = firstLabel.owner as INode
        const data = node.tag as MapObject
        const nodeSetup = NodeStyleSettings.getNodeSetupByMapObject(data)
        const fontBoldToggle = document.getElementById('font-bold') as HTMLInputElement
        fontBoldToggle.checked = nodeSetup.is_bold
        const fontItalicToggle = document.getElementById('font-italic') as HTMLInputElement
        fontItalicToggle.checked = nodeSetup.is_italic
      }
    }

    show () {
      // clearTimeout(this.hideTimer)
      this.container.style.display = 'block'

      // we hide the picker containers such that we don't need to update their position if new elements are added to
      // the toolbar
      this.hideAllPickerContainer()

      // show hide UI for nodes and/or labels
      this.updateItemUI()

      // maybe initialize some label UI elements to match the current label style
      this.updateLabelControlState()

      // place the contextual toolbar
      this.updateLocation()

      this.container.style.opacity = '1'
    }

    hide () {
      this.hideAllPickerContainer()
      this.container.style.opacity = '0'
      setTimeout(() => {
        this.container.style.display = 'none'
      }, 300)
    }

    updateLocation () {
      if (this.selectedItems.length === 0) {
        return
      }
      const width = this.container.clientWidth
      const height = this.container.clientHeight
      const zoom = this.graphComponent.zoom

      let dummyOwner: SimpleNode
      let labelModelParameter: ILabelModelParameter
      if (this.containsEdges && !this.containsShapeNodes) {
        // if only edges are selected, we want to use the first edge as position reference
        dummyOwner = this.selectedItems.find(item => IEdge.isInstance(item)) as SimpleNode
        labelModelParameter = this.edgeLabelModelParameter
      } else {
        // if nodes and edges are selected, we use the union of the node's bounding boxes as position reference
        dummyOwner = new SimpleNode({
          layout: this.getEnclosingRect()
        })
        labelModelParameter = this.nodeLabelModelParameter
      }

      // create a dummy label to let the LabelModelParameter compute the correct location
      const dummyLabel = new SimpleLabel(dummyOwner, '', labelModelParameter)
      if (labelModelParameter.supports(dummyLabel)) {
        dummyLabel.preferredSize = new Size(width / zoom, height / zoom)
        const newLayout = labelModelParameter.model.getGeometry(dummyLabel, labelModelParameter)
        this.setLocation(newLayout.anchorX, newLayout.anchorY - (height + 60) / zoom, width, height)
      }
    }

    getEnclosingRect () {
      const enclosingRect = new MutableRectangle()
      this.selectedItems.forEach(item => {
        if (INode.isInstance(item) || ILabel.isInstance(item)) {
          // we need the axis-parallel bounding rectangle, thus look out for oriented rectangles of the labels
          const bounds = IOrientedRectangle.isInstance(item.layout) ? item.layout.bounds : item.layout
          enclosingRect.setToUnion(enclosingRect, bounds)
        }
      })
      return enclosingRect
    }

    setLocation (x: number, y: number, width: number, height: number) {
      // Calculate the view coordinates since we have to place the div in the regular HTML coordinate space
      const viewPoint = this.graphComponent.toViewCoordinates(new Point(x, y))
      const gcSize = this.graphComponent.innerSize
      const padding = 15
      const left = Math.min(gcSize.width - width - padding, Math.max(padding, viewPoint.x))
      let top = Math.min(gcSize.height - height - padding, Math.max(padding, viewPoint.y))

      // todo: get offset

      top += 120
      this.container.style.left = `${left}px`
      this.container.style.top = `${top}px`
    }

    registerUpdateListeners () {
      this.graphComponent.addViewportChangedListener(() => {
        if (this.selectedItems.length > 0) {
          this.dirty = true
        }
      })
      this.graphComponent.graph.addNodeLayoutChangedListener(() => {
        if (this.selectedItems.length > 0) {
          this.dirty = true
        }
      })
      this.graphComponent.addUpdatedVisualListener(() => {
        if (this.selectedItems.length > 0 && this.dirty) {
          this.dirty = false
          this.updateLocation()
        }
      })
    }

    registerClickListeners () {
      this.bindAction('#color-picker', (e: React.MouseEvent<HTMLInputElement>) => this.showPickerContainer(e))
      this.bindAction('#shape-picker', (e: React.MouseEvent<HTMLInputElement>) => this.showPickerContainer(e))
      this.bindAction('#font-color-picker', (e: React.MouseEvent<HTMLInputElement>) => this.showPickerContainer(e))
      this.bindAction('#edge-color-picker', (e: React.MouseEvent<HTMLInputElement>) => this.showPickerContainer(e))
      this.bindAction('#edge-stroke-picker', (e: React.MouseEvent<HTMLInputElement>) => this.showPickerContainer(e))
      this.bindAction('#edge-type-picker', (e: React.MouseEvent<HTMLInputElement>) => this.showPickerContainer(e))
      this.bindAction('#source-arrow-picker', (e: React.MouseEvent<HTMLInputElement>) => {
        const cb = document.getElementById('target-arrow-picker') as HTMLInputElement
        cb.checked = false
        const target = e.target as HTMLInputElement
        const pickerContainer = document.getElementById(target.getAttribute('data-container-id'))
        this.removeClass(pickerContainer, 'target')
        this.showPickerContainer(e)
      })
      this.bindAction('#target-arrow-picker', (e: React.MouseEvent<HTMLInputElement>) => {
        const cb = document.getElementById('source-arrow-picker') as HTMLInputElement
        cb.checked = false
        const target = e.target as HTMLInputElement
        const pickerContainer = document.getElementById(target.getAttribute('data-container-id'))
        this.addClass(pickerContainer, 'target')
        this.showPickerContainer(e)
      })
      this.bindActions('#color-picker-colors > button', (e: React.MouseEvent<HTMLInputElement>) => {
        const target = e.target as HTMLInputElement
        const color = target.getAttribute('data-color')
        this.applyNodeSetup('background_color', color)
      })
      this.bindActions('#font-color-picker-colors > button', (e: React.MouseEvent<HTMLInputElement>) => {
        const target = e.target as HTMLInputElement
        const color = target.getAttribute('data-color')
        this.applyNodeSetup('color', color)
      })
      this.bindActions('#shape-picker-shapes > button', (e: React.MouseEvent<HTMLInputElement>) => {
        const target = e.target as HTMLInputElement
        const shape = target.getAttribute('data-shape')
        this.applyNodeSetup('shape', shape)
      })
      this.bindActions('#arrow-picker-types > button', (e: React.MouseEvent<HTMLInputElement>) => {
        const target = e.target as HTMLInputElement
        const arrowType = target.getAttribute('data-type')
        const isTarget = this.hasClass(target.parentElement, 'target')
        this.applyEdgeSetup((isTarget ? 'target_arrow' : 'source_arrow'), arrowType)
      })
      this.bindActions('#stroke-thickness > button', (e: React.MouseEvent<HTMLInputElement>) => {
        const target = e.target as HTMLInputElement
        const stroke = target.getAttribute('data-type')
        this.applyEdgeSetup('stroke', stroke)
      })
      this.bindActions('#stroke-types > button', (e: React.MouseEvent<HTMLInputElement>) => {
        const target = e.target as HTMLInputElement
        const style = target.getAttribute('data-type')
        this.applyEdgeSetup('style', style)
      })
      this.bindActions('#edge-colors > button', (e: React.MouseEvent<HTMLInputElement>) => {
        const target = e.target as HTMLInputElement
        const color = target.getAttribute('data-color')
        this.applyEdgeSetup('line_color', color)
      })
      this.bindAction('#font-bold', (e: React.MouseEvent<HTMLInputElement>) => {
        this.hideAllPickerContainer()
        const target = e.target as HTMLInputElement
        this.applyNodeSetup('is_bold', target.checked.toString())
      })
      this.bindAction('#font-italic', (e: React.MouseEvent<HTMLInputElement>) => {
        this.hideAllPickerContainer()
        const target = e.target as HTMLInputElement
        this.applyNodeSetup('is_italic', target.checked.toString())
      })
      this.bindAction('#decrease-font-size', () => {
        this.hideAllPickerContainer()
        this.changeFontSize(false)
      })
      this.bindAction('#increase-font-size', () => {
        this.hideAllPickerContainer()
        this.changeFontSize(true)
      })
      this.bindAction('#decrease-level', () => {
        this.hideAllPickerContainer()
        this.changeLevel(false)
      })
      this.bindAction('#increase-level', () => {
        this.hideAllPickerContainer()
        this.changeLevel(true)
      })
    }

    bindAction (selector: string, action: Function) {
      const element = document.querySelector(selector)
      if (!element) {
        return
      }
      element.addEventListener('click', e => {
        action(e)
      })
    }

    bindActions (selectors: string, action: Function) {
      const elements = document.querySelectorAll(selectors)
      if (!elements) {
        return
      }
      for (let i = 0; i < elements.length; i++) {
        const element = elements[i]
        element.addEventListener('click', e => {
          action(e)
        })
      }
    }

    addClass (e: Element, className: string) {
      const classes = e.getAttribute('class')
      if (classes === null || classes === '') {
        e.setAttribute('class', className)
      } else if (!this.hasClass(e, className)) {
        e.setAttribute('class', `${classes} ${className}`)
      }
      return e
    }

    removeClass (e: Element, className: string) {
      const classes = e.getAttribute('class')
      if (classes !== null && classes !== '') {
        if (classes === className) {
          e.setAttribute('class', '')
        } else {
          const result = classes
            .split(' ')
            .filter(s => s !== className)
            .join(' ')
          e.setAttribute('class', result)
        }
      }
      return e
    }

    hasClass (e: Element, className: string) {
      const classes = e.getAttribute('class')
      const r = new RegExp(`\\b${className}\\b`, '')
      return r.test(classes)
    }

    toggleClass (e: Element, className: string) {
      if (this.hasClass(e, className)) {
        this.removeClass(e, className)
      } else {
        this.addClass(e, className)
      }
      return e
    }
}
