import * as React from 'react'

import {
  ExteriorLabelModel,
  ExteriorLabelModelPosition,
  GraphComponent,
  INode,
  InteriorLabelModel,
  InteriorLabelModelPosition,
  Point,
  PolylineEdgeStyle,
  Rect,
  Size
} from 'yfiles/view-component'
import { ArcEdgeStyle } from 'yfiles/styles-other'
import { TextLabelStyle } from './styles/TextLabelStyle'
import { MyImageNodeStyle } from './styles/ImageNodeStyle'
import { MyShapeNodeStyle } from './styles/ShapeNodeStyle'
import { MyViewerInputMode } from './ViewInputMode'
import { EdgeStyleSettings } from './styles/EdgeStyleSettings'
import { EdgeMenu } from './menus/EdgeMenu'
import { TextMenu } from './menus/TextMenu'
import { TextNodeStyle } from './styles/TextNodeStyle'
import { CursorEditorInputMode } from './input_modes/CursorEditorInputMode'
import { ApiProps, NodeEditorInputMode } from './input_modes/NodeEditorInputMode'
import { EdgeEditorInputMode } from './input_modes/EdgeEditorInputMode'
import { NodeStyleSettings } from './styles/NodeStyleSettings'
import { CmdbObjectMenu } from './menus/CmdbObjectMenu'
import { ShapeMenu } from './menus/ShapeMenu'
import {
  DefaultGraph,
  FilteredGraphWrapper,
  ICommand,
  IEdge,
  IModelItem,
  OverviewInputMode
} from 'yfiles'
import { StructureView } from './StructureView'
import ContextualToolbar from './ContextualToolbar'
import { MapMenu } from './menus/MapMenu'
import { ImageMenu } from './menus/ImageMenu'
import { api } from './ajax'
import HtmlLabelStyle from "./styles/HtmlLabelStyle";

export interface ObjectType {
    id: number,
    title: string,
    icon: string,
}

export interface ShdMap {
    map_id: number,
    title: string,
}

export interface MapsComposeProps {
    data: string,
    base_url: string,
    base_path: string,
    language?: {[key: string]: string},
    api_url: string,
    session: string,
    api_key: string,
    object_types?: ObjectType[],
    maps?: ShdMap[],
}

export interface EdgeSetupProps {
    stroke: number,
    style: number,
    line_color: string,
    source_arrow_type: number,
    target_arrow_type: number,
}

export interface NodeSetupProps {
    family: string,
    size: number,
    color: string,
    background_color: string,
    is_italic: boolean,
    is_bold: boolean,
    shape_type: number,
}

interface MapsComposeState {
    is_editable: boolean,
    is_reloading: boolean,
    show_overview: boolean,
    show_structure: boolean,
    edit_mode: string,
    edge_setup: EdgeSetupProps,
    node_setup: NodeSetupProps,
}

interface MapData {
    map_id: number,
    title: string,
    map_objects: MapObject[],
    map_connection: MapConnection[],
    layerindex: string,
}

export interface MapObject {
    position: string,
    width: string,
    height: string,
    isys_obj__id?: number,
    mapobject_id: number,
    link?: string,
    title: string,
    is_title_custom?: string,
    textvisible: string,
    textalignment?: string,
    realizertype: string,
    labelposition: string,
    imagepath?: string,
    is_imagepath_custom?: string,
    image_file?: string,
    image_mime_type?: string,
    iconvisible?: string,
    shapetype: string,
    fillcolor?: string,
    fillcolor2?: string,
    linecolor?: string,
    labellinecolor?: string,
    textcolor: string,
    font?: string,
    status?: number,
    livestatus_instance_name?: string,
    livestatus_hostname?: string,
}

export interface MapConnection {
    mapconnection_id: number,
    source_shd_map_mapobject__id: number,
    target_shd_map_mapobject__id: number,
    textvisible: string,
    title: string,
    bends: string,
    sourcearrowtype: string,
    targetarrowtype: string,
    linecolor: string,
    linewidth: string,
    linestyle: string,
    realizertype: string,
}

export interface TransferObject {
    transfer_type: string,
    object_id: number,
    object_title?: string,
    icon?: string
    image_width?: number,
    image_height?: number,
}

interface NodeTag {
    title: string,
    textvisible: string,
    realizertype: number,
    shapetype: number,
    imagepath?: string,
    link?: string,
    isys_obj__id?: number,
    image_file?: string,
    width?: number,
    height?: number,
}

export class MapsCompose extends React.Component<MapsComposeProps, MapsComposeState> {
    private readonly div: React.RefObject<HTMLDivElement>;
    private readonly graphComponent: GraphComponent;
    private overviewComponent: GraphComponent;
    private structureView: StructureView;
    private contextualToolbar: ContextualToolbar;
    private readonly viewMode: MyViewerInputMode;
    private readonly cursorEditMode: CursorEditorInputMode;
    private readonly nodeEditMode: NodeEditorInputMode;
    private readonly edgeEditMode: EdgeEditorInputMode;
    private readonly apiProps: ApiProps;
    static readonly EDIT_MODE_CURSOR = 'cursor';
    static readonly EDIT_MODE_NODE = 'node';
    static readonly EDIT_MODE_SHAPE = 'shape';
    static readonly EDIT_MODE_MAP = 'map';
    static readonly EDIT_MODE_IMAGE = 'image';
    static readonly EDIT_MODE_TEXT = 'text';
    static readonly EDIT_MODE_EDGE = 'edge';
    private nodeLayerIndices: number[];

    constructor (props: MapsComposeProps) {
      super(props)
      this.state = {
        is_editable: false,
        is_reloading: false, // true,
        show_overview: false,
        show_structure: false,
        edit_mode: MapsCompose.EDIT_MODE_CURSOR,
        edge_setup: {
          stroke: EdgeStyleSettings.EDGE_STROKE_S,
          style: EdgeStyleSettings.EDGE_STYLE_SOLID,
          line_color: '#000000',
          source_arrow_type: EdgeStyleSettings.ARROW_TYPE_NONE,
          target_arrow_type: EdgeStyleSettings.ARROW_TYPE_DEFAULT
        },
        node_setup: {
          family: 'Dialog',
          size: 14,
          color: '#000000',
          background_color: '#7784AA',
          is_bold: false,
          is_italic: false,
          shape_type: null
        }
      }

      this.setDefaultNodeStyle = this.setDefaultNodeStyle.bind(this)
      this.setDefaultEdgeStyle = this.setDefaultEdgeStyle.bind(this)
      this.renderGraph = this.renderGraph.bind(this)
      this.handleMapEditable = this.handleMapEditable.bind(this)
      this.handlePageReload = this.handlePageReload.bind(this)
      this.handleFitBounds = this.handleFitBounds.bind(this)
      this.handleShowOverview = this.handleShowOverview.bind(this)
      this.handleShowStructure = this.handleShowStructure.bind(this)
      this.handleEditMode = this.handleEditMode.bind(this)

      this.updateEdgeSetup = this.updateEdgeSetup.bind(this)
      this.updateNodeSetup = this.updateNodeSetup.bind(this)
      this.updateNodeLevel = this.updateNodeLevel.bind(this)
      this.updateHiddenNode = this.updateHiddenNode.bind(this)

      this.handleDragOver = this.handleDragOver.bind(this)
      this.handleDrop = this.handleDrop.bind(this)

      this.div = React.createRef<HTMLDivElement>()
      this.graphComponent = new GraphComponent('#yGraph')

      // create a filtered graph
      const fullGraph = new DefaultGraph()
      const nodePredicate = (node: INode) => !node.tag || !node.tag.filtered
      const edgePredicate = (edge: IEdge) => !edge.tag || !edge.tag.filtered
      this.graphComponent.graph = new FilteredGraphWrapper(fullGraph, nodePredicate, edgePredicate)

      const mapData: MapData = JSON.parse(this.props.data)
      this.viewMode = new MyViewerInputMode(this.props.base_path)
      this.apiProps = {
        graph_component: this.graphComponent,
        api_key: this.props.api_key,
        api_url: this.props.api_url,
        session: this.props.session,
        map_id: mapData.map_id
      }
      this.cursorEditMode = new CursorEditorInputMode()
      this.nodeEditMode = new NodeEditorInputMode(this.apiProps, {
        base_url: this.props.base_url,
        node_setup: this.state.node_setup
      }, this.updateNodeSetup)
      this.edgeEditMode = new EdgeEditorInputMode(this.apiProps, this.updateEdgeSetup, this.state.edge_setup)
      this.graphComponent.inputMode = this.viewMode

      this.setDefaultNodeStyle(MapsCompose.EDIT_MODE_NODE)
      this.setDefaultEdgeStyle(true)
    }

    componentDidMount () {
      // Append the GraphComponent to the DOM
      this.div.current!.appendChild(this.graphComponent.div)
      this.renderGraph()

      // add overview
      this.overviewComponent = new GraphComponent('overviewComponent')
      const overviewInputMode = new OverviewInputMode()
      overviewInputMode.canvasComponent = this.graphComponent
      this.overviewComponent.inputMode = overviewInputMode

      // sets the overview graph and fit the overview graph bounds
      this.overviewComponent.graph = this.graphComponent.graph

      // updates the overview component then show the overview graph
      this.overviewComponent.updateVisualAsync().then(() => {
        this.overviewComponent.fitGraphBounds()
      })

      // add structure view
      const map: MapData = JSON.parse(this.props.data)
      this.structureView = new StructureView('#structureComponent', this.props.language, this.updateHiddenNode, map.title)
      this.structureView.setGraph(this.graphComponent.graph)
      this.structureView.onElementClicked = (node: INode) => {
        const viewNode = this.graphComponent.graph.foldingView
          ? this.graphComponent.graph.foldingView.getViewItem(node)
          : node
        if (viewNode) {
          this.graphComponent.currentItem = viewNode
          this.graphComponent.selection.clear()
          this.graphComponent.selection.setSelected(viewNode, true)
          ICommand.ZOOM_TO_CURRENT_ITEM.execute(null, this.graphComponent)
        }
      }
      this.nodeEditMode.setStructureView(this.structureView)

      // add contextual toolbar
      this.initContextualToolbar()
    }

    initContextualToolbar () {
      this.contextualToolbar = new ContextualToolbar(
        this.graphComponent,
        window.document.getElementById('contextualToolbar'),
        {
          onUpdateNodeSetup: this.updateNodeSetup,
          onUpdateEdgeSetup: this.updateEdgeSetup,
          onUpdateNodeLevel: this.updateNodeLevel,
          edgeSetup: this.state.edge_setup
        }
      )

      // when an item is right clicked
      this.nodeEditMode.addItemRightClickedListener((src, args) => {
        const isSelected = this.graphComponent.selection.filter((item: IModelItem) => item == args.item).size > 0
        if (!isSelected) {
          this.graphComponent.selection.clear()
          this.graphComponent.selection.setSelected(args.item, true)
        }
        this.contextualToolbar.setSelectedItems([args.item])
      })

      this.edgeEditMode.addItemRightClickedListener((src, args) => {
        const isSelected = this.graphComponent.selection.filter((item: IModelItem) => item == args.item).size > 0
        if (!isSelected) {
          this.graphComponent.selection.clear()
          this.graphComponent.selection.setSelected(args.item, true)
        }
        this.contextualToolbar.setSelectedItems([args.item])
      })

      // if an item is deselected or deleted, we remove that element from the selectedItems
      this.graphComponent.selection.addItemSelectionChangedListener((src, args) => {
        if (!args.itemSelected) {
          this.contextualToolbar.setSelectedItems([])
        }
      })
    }

    setDefaultNodeStyle (mode: string) {
      const graph = this.graphComponent.graph
      graph.nodeDefaults.size = new Size(64, 64)
      if (mode == MapsCompose.EDIT_MODE_NODE || mode == MapsCompose.EDIT_MODE_MAP || mode == MapsCompose.EDIT_MODE_IMAGE) {
        graph.nodeDefaults.style = new MyImageNodeStyle(this.props.base_url)

        const labelModel = new ExteriorLabelModel({ insets: 5 })
        graph.nodeDefaults.labels.layoutParameter = labelModel.createParameter(ExteriorLabelModelPosition.SOUTH)
        graph.nodeDefaults.labels.style = new TextLabelStyle(this.state.node_setup)
      }
      if (mode == MapsCompose.EDIT_MODE_SHAPE) {
        graph.nodeDefaults.style = new MyShapeNodeStyle()

        const labelModel = new ExteriorLabelModel({ insets: 5 })
        graph.nodeDefaults.labels.layoutParameter = labelModel.createParameter(ExteriorLabelModelPosition.SOUTH)
        graph.nodeDefaults.labels.style = new TextLabelStyle(this.state.node_setup)
      }
      if (mode == MapsCompose.EDIT_MODE_TEXT) {
        graph.nodeDefaults.style = new TextNodeStyle()

        const labelModel = new InteriorLabelModel()
        this.graphComponent.graph.nodeDefaults.labels.layoutParameter = labelModel.createParameter(InteriorLabelModelPosition.CENTER)
      }
    }

    setDefaultEdgeStyle (isPolyline: boolean) {
      const e = this.state.edge_setup
      const options = {
        stroke: EdgeStyleSettings.getEdgeStroke(e.stroke, e.style, e.line_color),
        sourceArrow: EdgeStyleSettings.getEdgeArrow(e.source_arrow_type, e.stroke, e.line_color),
        targetArrow: EdgeStyleSettings.getEdgeArrow(e.target_arrow_type, e.stroke, e.line_color)
      }
      if (isPolyline) {
        this.graphComponent.graph.edgeDefaults.style = new PolylineEdgeStyle(options)
      } else {
        this.graphComponent.graph.edgeDefaults.style = new ArcEdgeStyle(options)
      }
    }

    renderGraph () {
      // clear graph
      this.graphComponent.graph.clear()

      const mapData: MapData = JSON.parse(this.props.data)
      const graph = this.graphComponent.graph
      const nodeMap:{[key: number]: INode} = {}

      mapData.map_objects.forEach((obj) => {
        const pos = obj.position.split(',')
        let x = parseInt(pos[0])
        let y = parseInt(pos[1])
        const w = parseInt(obj.width)
        const h = parseInt(obj.height)

        // re-calculate position from node-center (SM-VIEW) to top-left-corner (yFiles HTML)
        x = x - (w / 2)
        y = y - (h / 2)

        const node: INode = graph.createNode(new Rect(x, y, w, h))
        node.tag = obj
        nodeMap[obj.mapobject_id] = node

        const label = graph.addLabel(node, obj.title)
        if (obj.realizertype == '2' || obj.realizertype == '1') {
          // shape node or text node
          if (obj.title.includes('</')) {
            graph.setStyle(label, new HtmlLabelStyle(this.state.node_setup))
          } else {
            graph.setStyle(label, new TextLabelStyle(this.state.node_setup))
          }

          const labelModel = new InteriorLabelModel()
          const labelPos = InteriorLabelModelPosition.CENTER
          graph.setLabelLayoutParameter(label, labelModel.createParameter(labelPos))
        }
        if (obj.realizertype == '0') {
          // image node
          // graph.setStyle(node, new MyImageNodeStyle(this.props.base_url));

          if (obj.title.includes('</')) {
            graph.setStyle(label, new HtmlLabelStyle(this.state.node_setup))
          } else {
            graph.setStyle(label, new TextLabelStyle())
          }

          // let labelPos = MapsCompose.getLabelPosition(obj.labelposition);
          const labelPos = NodeStyleSettings.getPosition(parseInt(obj.labelposition))
          const labelModel = new ExteriorLabelModel({ insets: 5 })
          graph.setLabelLayoutParameter(label, labelModel.createParameter(labelPos))
        }
        graph.setStyle(node, NodeStyleSettings.getNodeStyle(obj, this.props.base_url))

        if (obj.textvisible == '0') {
          // remove label if text is not visible
          graph.remove(label)
        }
      })

      if (!mapData.map_connection) {
        return
      }

      mapData.map_connection.forEach((obj) => {
        const sourceNode: INode = nodeMap[obj.source_shd_map_mapobject__id]
        const targetNode: INode = nodeMap[obj.target_shd_map_mapobject__id]
        const edge = graph.createEdge(sourceNode, targetNode)
        edge.tag = obj

        if (obj.textvisible && obj.textvisible == '1' && obj.title && obj.title != '') {
          const label = graph.addLabel(edge, obj.title)
          const labelModel = new InteriorLabelModel()
          const labelPos = InteriorLabelModelPosition.CENTER

          graph.setStyle(label, new TextLabelStyle())
          graph.setLabelLayoutParameter(label, labelModel.createParameter(labelPos))
        }

        if (obj.bends != '') {
          const bends = obj.bends.split(';')
          bends.forEach((b) => {
            const coord = b.split(',')
            const x = parseInt(coord[0])
            const y = parseInt(coord[1])
            graph.addBend(edge, new Point(x, y))
          })
        }
        graph.setStyle(edge, EdgeStyleSettings.getEdgeStyle(obj))
      })

      // set layerindex of each node
      let mapLayerIndex = mapData.layerindex.split(';')
      this.nodeLayerIndices = []
      mapLayerIndex.forEach(objectId => {
        const mapObjectId = parseInt(objectId)
        if (! nodeMap[mapObjectId]) {
          return
        }
        const node: INode = nodeMap[mapObjectId]
        this.graphComponent.graphModelManager.getCanvasObject(node).toFront()
        this.nodeLayerIndices.push(mapObjectId)
      })
      this.graphComponent.fitGraphBounds()
    }

    handleMapEditable () {
      const checked = !this.state.is_editable
      this.setState((prevState) => ({ ...prevState, is_editable: checked }))
      this.graphComponent.inputMode = checked ? this.cursorEditMode : this.viewMode
    }

    handlePageReload () {
      const checked = !this.state.is_reloading
      this.setState((prevState) => ({ ...prevState, is_reloading: checked }))

      // setTimeout(function () {
      //     if (! this.state.is_reloading) {
      //         return;
      //     }
      //     window.location.href = window.location.toString();
      // }, 60000);
    }

    handleFitBounds () {
      this.graphComponent.fitContent()
    }

    handleShowOverview () {
      const checked = !this.state.show_overview
      this.setState((prevState) => ({ ...prevState, show_overview: checked }))
    }

    handleShowStructure () {
      const checked = !this.state.show_structure
      this.setState((prevState) => ({ ...prevState, show_structure: checked }))
    }

    handleEditMode (event: React.MouseEvent<HTMLButtonElement>) {
      this.graphComponent.selection.clear()

      const editMode = event.currentTarget.value
      this.setState((prevState) => ({ ...prevState, edit_mode: editMode }))

      switch (editMode) {
        case MapsCompose.EDIT_MODE_CURSOR:
          this.graphComponent.inputMode = this.cursorEditMode
          break
        case MapsCompose.EDIT_MODE_NODE:
        case MapsCompose.EDIT_MODE_SHAPE:
        case MapsCompose.EDIT_MODE_TEXT:
        case MapsCompose.EDIT_MODE_MAP:
        case MapsCompose.EDIT_MODE_IMAGE:
          this.nodeEditMode.allowNodeCreation(editMode == MapsCompose.EDIT_MODE_TEXT)
          this.graphComponent.inputMode = this.nodeEditMode
          this.setDefaultNodeStyle(editMode)
          break
        case MapsCompose.EDIT_MODE_EDGE:
          this.graphComponent.inputMode = this.edgeEditMode
          break
      }
    }

    updateEdgeSetup (edgeSetup: EdgeSetupProps, updateSelectedEdges?: boolean) {
      console.debug('update edge STATE')
      this.setState((prevState) => ({ ...prevState, edge_setup: edgeSetup }), () => {
        if (updateSelectedEdges) {
          // change settings of selected edges
          this.edgeEditMode.updateSelectedEdges(edgeSetup)
        }

        // update edge editor props
        this.edgeEditMode.updateEditorProps(edgeSetup)

        // always update default edge style to fit with displayed edge menu
        this.setDefaultEdgeStyle(this.graphComponent.graph.edgeDefaults.style instanceof PolylineEdgeStyle)
      })
    }

    updateNodeSetup (nodeSetup: NodeSetupProps, updateSelectedNodes?: boolean) {
      console.debug('update node STATE')
      console.debug(nodeSetup)
      this.setState((prevState) => ({ ...prevState, node_setup: nodeSetup }), () => {
        // change settings of selected nodes
        if (updateSelectedNodes) {
          this.nodeEditMode.updateSelectedNodes(nodeSetup)
        }

        // update node editor props
        this.nodeEditMode.updateNodeSetup(nodeSetup)

        // always update default node style to fit with displayed font menu
        this.setDefaultNodeStyle(this.state.edit_mode)
      })
    }

    updateHiddenNode (node: INode, isHidden: boolean) {
      node.tag.filtered = isHidden

      // re-evaluate the filter predicates to actually hide the items
      const filteredGraph = this.graphComponent.graph as FilteredGraphWrapper
      filteredGraph.nodePredicateChanged()
    }

    updateNodeLevel (node: INode, doIncrease: boolean) {
      const data = node.tag as MapObject
      const id = data.mapobject_id
      const pos = this.nodeLayerIndices.indexOf(id)
      let newPos = doIncrease ? pos + 1 : pos - 1
      newPos = newPos < 0 ? 0 : newPos

      // re-order node indices
      if (newPos >= this.nodeLayerIndices.length) {
        let k = newPos - this.nodeLayerIndices.length + 1
        while (k--) {
          this.nodeLayerIndices.push(undefined)
        }
      }
      this.nodeLayerIndices.splice(newPos, 0, this.nodeLayerIndices.splice(pos, 1)[0])

      // update map
      const map: MapData = JSON.parse(this.props.data)
      api('shd.map.update',
        {
          map_id: map.map_id,
          layerindex: JSON.stringify(this.nodeLayerIndices)
        },
        this.apiProps).then(() => {
        if (doIncrease) {
          this.graphComponent.graphModelManager.getCanvasObject(node).raise()
        } else {
          this.graphComponent.graphModelManager.getCanvasObject(node).lower()
        }
      })
    }

    handleDragOver (evt: React.DragEvent<HTMLDivElement>) {
      evt.preventDefault()
    }

    handleDrop (evt: React.DragEvent<HTMLDivElement>) {
      evt.preventDefault()
      console.debug('handle drop')

      const transfer: TransferObject = JSON.parse(evt.dataTransfer.getData('text/json'))
      let style
      let imagePath = null
      let title = null
      const tag: NodeTag = {
        title: null,
        textvisible: '1',
        realizertype: 0,
        shapetype: 0
      }

      switch (transfer.transfer_type) {
        case 'cmdb-object':
          style = new MyImageNodeStyle(this.props.base_url)
          title = transfer.object_title ? transfer.object_title : 'New Node'
          imagePath = this.props.base_url + transfer.icon
          tag.isys_obj__id = transfer.object_id
          break
        case 'map':
          style = new MyImageNodeStyle(this.props.base_url)
          title = transfer.object_title ? transfer.object_title : 'Linked Map'
          imagePath = this.props.base_url + transfer.icon
          tag.link = transfer.object_id.toString()
          break
        case 'shape':
          style = new MyShapeNodeStyle()
          tag.shapetype = transfer.object_id
          tag.realizertype = 1
          break
        case 'image':
          style = new MyImageNodeStyle(this.props.base_url)
          title = transfer.object_title ? transfer.object_title : 'Image'
          tag.textvisible = '0'
          imagePath = transfer.object_title
          tag.image_file = transfer.icon
          tag.width = transfer.image_width ? transfer.image_width : 64
          tag.height = transfer.image_height ? transfer.image_height : 64
          break
        default:
          return
      }
      tag.imagepath = imagePath
      tag.title = title

      // get drop location
      let point = new Point(evt.clientX, evt.clientY)
      point = this.graphComponent.toWorldFromPage(point)
      const node: INode = this.graphComponent.graph.createNodeAt(point, style, tag)

      // update node size if needed
      if (tag.width) {
        this.graphComponent.graph.setNodeLayout(
          node,
          new Rect(node.layout.topLeft, new Size(tag.width, tag.height))
        )
      }

      // save node
      this.nodeEditMode.setupNewNode(node)
    }

    render () {
      let toolMenu = null
      let menu = null
      if (this.state.edit_mode == MapsCompose.EDIT_MODE_NODE) {
        toolMenu = <CmdbObjectMenu api_props={this.apiProps}
          base_url={this.props.base_url}
          object_types={this.props.object_types}
          language={this.props.language} />
      }
      if (this.state.edit_mode == MapsCompose.EDIT_MODE_SHAPE) {
        toolMenu = <ShapeMenu node_setup={this.state.node_setup}
          onNodeSetupChange={this.updateNodeSetup} />
      }
      if (this.state.edit_mode == MapsCompose.EDIT_MODE_TEXT) {
        toolMenu = <TextMenu node_setup={this.state.node_setup}
          onNodeSetupChange={this.updateNodeSetup}
          language={this.props.language} />
      }
      if (this.state.edit_mode == MapsCompose.EDIT_MODE_IMAGE) {
        toolMenu = <ImageMenu base_url={this.props.base_url}
          api_props={this.apiProps}
          language={this.props.language} />
      }
      if (this.state.edit_mode == MapsCompose.EDIT_MODE_MAP) {
        toolMenu = <MapMenu maps={this.props.maps}
          language={this.props.language} />
      }
      if (this.state.edit_mode == MapsCompose.EDIT_MODE_EDGE) {
        toolMenu = <EdgeMenu base_url={this.props.base_url}
          edge_setup={this.state.edge_setup}
          onEdgeSetupChange={this.updateEdgeSetup}
          language={this.props.language} />
      }
      if (this.state.is_editable) {
        const className = this.state.show_overview || this.state.show_structure ? 'menu menu-with-overview' : 'menu'
        menu = <div className={className}>
          <div className="tools">
            <button className={this.state.edit_mode == MapsCompose.EDIT_MODE_CURSOR ? 'btn selected' : 'btn'}
              type="button"
              onClick={this.handleEditMode}
              value={MapsCompose.EDIT_MODE_CURSOR}>
              {this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_CURSOR}
            </button>
            <button className={this.state.edit_mode == MapsCompose.EDIT_MODE_NODE ? 'btn selected' : 'btn'}
              type="button"
              onClick={this.handleEditMode}
              value={MapsCompose.EDIT_MODE_NODE}>
              {this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_NODE}
            </button>
            <button className={this.state.edit_mode == MapsCompose.EDIT_MODE_SHAPE ? 'btn selected' : 'btn'}
              type="button"
              onClick={this.handleEditMode}
              value={MapsCompose.EDIT_MODE_SHAPE}>
              {this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_SHAPE}
            </button>
            <button className={this.state.edit_mode == MapsCompose.EDIT_MODE_TEXT ? 'btn selected' : 'btn'}
              type="button"
              onClick={this.handleEditMode}
              value={MapsCompose.EDIT_MODE_TEXT}>
              {this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_TEXT}
            </button>
            <button className={this.state.edit_mode == MapsCompose.EDIT_MODE_IMAGE ? 'btn selected' : 'btn'}
              type="button"
              onClick={this.handleEditMode}
              value={MapsCompose.EDIT_MODE_IMAGE}>
              {this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_IMAGE}
            </button>
            <button className={this.state.edit_mode == MapsCompose.EDIT_MODE_MAP ? 'btn selected' : 'btn'}
              type="button"
              onClick={this.handleEditMode}
              value={MapsCompose.EDIT_MODE_MAP}>
              {this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_MAP}
            </button>
            <button className={this.state.edit_mode == MapsCompose.EDIT_MODE_EDGE ? 'btn selected' : 'btn'}
              type="button"
              onClick={this.handleEditMode}
              value={MapsCompose.EDIT_MODE_EDGE}>
              {this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_EDGE}
            </button>
          </div>
          {toolMenu}
        </div>
      }

      const iconUrl = this.props.base_url + 'images/icons/silk/'
      const overviewStyle = this.state.show_overview ? {} : { display: 'none' }
      const structureClass = this.state.show_overview ? 'structure-container-with-overview' : ''
      const structureStyle = this.state.show_structure ? {} : { display: 'none' }
      const map: MapData = JSON.parse(this.props.data)
      return <div style={{ height: '100%' }}>
        <div className="settings">
          <ul>
            <li>
              <div className={this.state.show_overview ? 'navbar_item selected' : 'navbar_item'}
                onClick={this.handleShowOverview}>
                <img src={iconUrl + 'chart_organisation.png'}
                  alt={this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__OVERVIEW} width="15px" height="15px" />
                <label>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__OVERVIEW}</label>
              </div>
              <div className="overview-container" style={overviewStyle}>
                <div id="overviewComponent">&nbsp;</div>
              </div>
            </li>
            <li>
              <div className={this.state.show_structure ? 'navbar_item selected' : 'navbar_item'}
                onClick={this.handleShowStructure}>
                <img src={iconUrl + 'outline.png'}
                  alt={this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__STRUCTURE} width="15px" height="15px" />
                <label>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__STRUCTURE}</label>
              </div>
              <div className={'structure-container ' + structureClass} style={structureStyle}>
                <div id="structureComponent" className="structure-view-container">&nbsp;</div>
              </div>
            </li>
            <li>
              <div className={this.state.is_editable ? 'navbar_item selected' : 'navbar_item'}
                onClick={this.handleMapEditable}>
                <img src={iconUrl + 'page_edit.png'}
                  alt={this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MAP} width="15px" height="15px" />
                <label>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MAP}</label>
              </div>
            </li>
            <li>
              <div className="navbar_item"
                onClick={this.handleFitBounds}>
                <img src={iconUrl + 'arrow_out.png'}
                  alt="" width="15px" height="15px" />
                <label>Fit Bounds</label>
              </div>
            </li>
            <li>
              <div className={this.state.is_reloading ? 'navbar_item selected' : 'navbar_item'}
                onClick={this.handlePageReload}>
                <img src={iconUrl + 'arrow_refresh.png'}
                  alt={this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__AUTO_RELOAD_PAGE} width="15px" height="15px" />
                <label>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__AUTO_RELOAD_PAGE}</label>
              </div>
            </li>
          </ul>
        </div>
        <div className="map-name">
          <h2>{map.title}</h2>
        </div>
        {menu}
        <div className="graph" onDragOver={this.handleDragOver} onDrop={this.handleDrop}>
          <div className="graph-component-container" ref={this.div} />
        </div>
      </div>
    }
}
