'use strict'

import 'yfiles/view-layout-bridge.js'
import { GraphItemTypes } from 'yfiles/view-component'
import { EdgeSetupProps, MapConnection } from '../MapsCompose'
import { api } from '../ajax'
import {
  GraphEditorInputMode, IEdge,
  IModelItem,
  ItemClickedEventArgs,
  PortEventArgs
} from 'yfiles'
import { EdgeCreationInputMode } from './EdgeCreationInputMode'
import { ApiProps } from './NodeEditorInputMode'
import { EdgeStyleSettings } from '../styles/EdgeStyleSettings'
import { EdgeMovingInputMode } from './EdgeMovingInputMode'

export class EdgeEditorInputMode extends GraphEditorInputMode {
    private readonly apiProps: ApiProps;
    private editorProps: EdgeSetupProps;
    private readonly onLeftClick: Function;

    constructor (apiProps: ApiProps, onLeftClick: Function, editorProps: EdgeSetupProps) {
      super({
        clickableItems: GraphItemTypes.NODE | GraphItemTypes.EDGE | GraphItemTypes.EDGE_LABEL | GraphItemTypes.BEND,
        clickSelectableItems: GraphItemTypes.EDGE | GraphItemTypes.EDGE_LABEL | GraphItemTypes.BEND,
        createEdgeInputMode: new EdgeCreationInputMode(apiProps, editorProps),
        moveInputMode: new EdgeMovingInputMode(apiProps),
        focusableItems: GraphItemTypes.EDGE | GraphItemTypes.NODE_LABEL,
        selectableItems: GraphItemTypes.EDGE | GraphItemTypes.EDGE_LABEL | GraphItemTypes.BEND,
        showHandleItems: GraphItemTypes.EDGE | GraphItemTypes.EDGE_LABEL,
        marqueeSelectableItems: GraphItemTypes.NONE,
        movableItems: GraphItemTypes.EDGE | GraphItemTypes.EDGE_LABEL | GraphItemTypes.BEND,
        deletableItems: GraphItemTypes.EDGE | GraphItemTypes.BEND,
        labelEditableItems: GraphItemTypes.LABEL_OWNER | GraphItemTypes.LABEL | GraphItemTypes.EDGE,
        allowClipboardOperations: false,
        allowGroupingOperations: false,
        allowDuplicate: false,
        allowAddLabel: true,
        allowEditLabel: true,
        allowCreateNode: false,
        allowCreateEdge: true,
        allowCreateBend: true
      })
      this.apiProps = apiProps
      this.editorProps = editorProps
      this.onLeftClick = onLeftClick
    }

    updateEditorProps (props: EdgeSetupProps) {
      this.editorProps = props
      const mode = this.createEdgeInputMode as EdgeCreationInputMode
      mode.updateEditorProps(props)
    }

    updateSelectedEdges (props: EdgeSetupProps) {
      if (this.apiProps.graph_component.selection.selectedEdges.size <= 0) {
        return
      }

      this.apiProps.graph_component.selection.selectedEdges.forEach((edge: IEdge) => {
        const connection: MapConnection = edge.tag as MapConnection
        connection.linewidth = props.stroke.toString()
        connection.linestyle = props.style.toString()
        connection.linecolor = props.line_color
        connection.sourcearrowtype = props.source_arrow_type.toString()
        connection.targetarrowtype = props.target_arrow_type.toString()
        edge.tag = connection

        // repaint edge
        this.apiProps.graph_component.graph.setStyle(edge, EdgeStyleSettings.getEdgeStyle(connection))

        // update edge
        api('shd.map_connection.update',
          {
            mapconnection_id: connection.mapconnection_id,
            ...connection
          },
          this.apiProps).then(() => {
        })
      })
    }

    onItemLeftClicked (evt: ItemClickedEventArgs<IModelItem>) {
      super.onItemLeftClicked(evt)

      if (!evt.item.tag || !('mapconnection_id' in evt.item.tag)) {
        // clicked object is not an edge
        return
      }

      // load edge settings
      const connection = evt.item.tag as MapConnection
      const edgeSetup = EdgeStyleSettings.getEdgeSetupByMapConnection(connection)

      // update global edge setup with these settings
      this.onLeftClick(edgeSetup)
    }

    onPortAdded (evt: PortEventArgs) {
      console.debug('on port added')
    }

    deleteSelection () {
      this.graphComponent.selection.selectedEdges.forEach((edge) => {
        const mapConnection: MapConnection = edge.tag as MapConnection
        api('shd.map_connection.delete',
          {
            mapconnection_id: mapConnection.mapconnection_id
          },
          this.apiProps).then(() => {
          console.debug('edge successful deleted')
        })
      })
      super.deleteSelection()
    }
}
