'use strict'

import 'yfiles/view-layout-bridge.js'
import { ExteriorLabelModelPosition, GraphComponent, GraphItemTypes } from 'yfiles/view-component'
import { NodeSetupProps, MapObject, MapConnection } from '../MapsCompose'
import { api } from '../ajax'
import {
  GraphEditorInputMode,
  ILabel,
  IModelItem,
  INode,
  ItemClickedEventArgs,
  ItemEventArgs,
  LabelEventArgs
} from 'yfiles'
import { ChangeNodeSizeInputMode } from './ChangeNodeSizeInputMode'
import { ChangeNodePositionInputMode } from './ChangeNodePositionInputMode'
import { LabelPositionInputMode } from './LabelPositionInputMode'
import { MyShapeNodeStyle } from '../styles/ShapeNodeStyle'
import { MyImageNodeStyle } from '../styles/ImageNodeStyle'
import { TextNodeStyle } from '../styles/TextNodeStyle'
import { NodeStyleSettings } from '../styles/NodeStyleSettings'
import { StructureView } from '../StructureView'

export interface ApiProps {
    graph_component: GraphComponent,
    api_key: string,
    api_url: string,
    session: string,
    map_id: number,
}

interface NodeEditorProps {
    base_url: string,
    node_setup: NodeSetupProps,
}

export class NodeEditorInputMode extends GraphEditorInputMode {
    private readonly apiProps: ApiProps;
    private editorProps: NodeEditorProps;
    private readonly onLeftClick: Function;
    private structureView: StructureView;

    constructor (apiProps: ApiProps, editorProps: NodeEditorProps, onLeftClick: Function) {
      super({
        clickableItems: GraphItemTypes.NODE | GraphItemTypes.NODE_LABEL,
        clickSelectableItems: GraphItemTypes.NODE | GraphItemTypes.NODE_LABEL,
        // clickInputMode: new HighlightClickInputMode(graphComponent, 0),
        focusableItems: GraphItemTypes.NODE | GraphItemTypes.NODE_LABEL,
        selectableItems: GraphItemTypes.NODE | GraphItemTypes.NODE_LABEL,
        showHandleItems: GraphItemTypes.NODE | GraphItemTypes.NODE_LABEL,
        marqueeSelectableItems: GraphItemTypes.NONE,
        movableItems: GraphItemTypes.NODE | GraphItemTypes.NODE_LABEL,
        moveInputMode: new ChangeNodePositionInputMode(apiProps),
        handleInputMode: new ChangeNodeSizeInputMode(apiProps),
        deletableItems: GraphItemTypes.NODE,
        // labelDropInputMode : LabelDropInputMode
        labelEditableItems: GraphItemTypes.LABEL_OWNER | GraphItemTypes.LABEL | GraphItemTypes.NODE,
        moveLabelInputMode: new LabelPositionInputMode(apiProps),
        // nodeDropInputMode : DropInputMode('text/json'),
        allowClipboardOperations: false,
        allowGroupingOperations: false,
        allowDuplicate: false,
        allowAddLabel: true,
        allowEditLabel: true,
        allowCreateNode: false,
        allowCreateEdge: false,
        allowCreateBend: false
      })
      this.apiProps = apiProps
      this.editorProps = editorProps
      this.onLeftClick = onLeftClick
    }

    allowNodeCreation (allow: boolean) {
      this.allowCreateNode = allow
    }

    updateNodeSetup (props: NodeSetupProps) {
      this.editorProps.node_setup = props
    }

    setStructureView (structureView: StructureView) {
      this.structureView = structureView
    }

    updateStructureView () {
      if (this.structureView) {
        this.structureView.buildStructure()
      }
    }

    updateSelectedNodes (props: NodeSetupProps) {
      if (this.apiProps.graph_component.selection.selectedNodes.size <= 0) {
        return
      }

      this.apiProps.graph_component.selection.selectedNodes.forEach(node => {
        const data: MapObject = node.tag as MapObject
        console.debug('change font of selected node: ' + data.title)

        data.font = NodeStyleSettings.getFontStringByNodeSetup(props)
        data.textcolor = props.color
        data.fillcolor = props.background_color
        if (data.realizertype == '1') {
          data.shapetype = props.shape_type.toString()
        }
        node.tag = data

        // repaint node
        this.apiProps.graph_component.graph.setStyle(node, NodeStyleSettings.getNodeStyle(data, this.editorProps.base_url))

        // update node
        api('shd.map_object.update',
          {
            mapobject_id: data.mapobject_id,
            font: data.font,
            textcolor: data.textcolor,
            fillcolor: data.fillcolor,
            shapetype: data.shapetype
          },
          this.apiProps).then(() => {
        })
      })
    }

    onItemLeftClicked (evt: ItemClickedEventArgs<IModelItem>) {
      super.onItemLeftClicked(evt)

      if (!evt.item.tag || !('mapobject_id' in evt.item.tag)) {
        // clicked object is not a node
        return
      }

      // load node settings
      const mapObject: MapObject = evt.item.tag as MapObject
      const nodeSetup = NodeStyleSettings.getNodeSetupByMapObject(mapObject)
      if (!nodeSetup) {
        return
      }

      // update global edge setup with these settings
      this.onLeftClick(nodeSetup)
    }

    onLabelTextEdited (label: ILabel, text: string) {
      return super.onLabelTextEdited(label, text).then((res): boolean => {
        const mapObject: MapObject = label.owner.tag as MapObject
        const textVisible = text.length > 0
        api('shd.map_object.update',
            {
              mapobject_id: mapObject.mapobject_id,
              title: text,
              is_title_custom: true,
              textvisible: textVisible
            },
            this.apiProps).then(() => {
          this.updateStructureView()
        })
        return res
      })
    }

    onLabelAdded (evt: LabelEventArgs) {
      console.debug('on label added')
    }

    onNodeCreated (evt: ItemEventArgs<INode>) {
      super.onNodeCreated(evt)

      // setup new text node
      evt.item.tag = {
        fillcolor: '#FFFFFF',
        title: 'Text'
      }

      this.setupNewNode(evt.item)
    }

    setupNewNode (node: INode) {
      // note: if the node has already a tag, it was dropped on the map
      // otherwise it was created by a click on the map
      let realizertype = 0
      const defaults = node.tag
      let title = defaults && defaults.title ? defaults.title : 'New Node'
      let labelLineColor = null

      // get settings of current style
      let style = null
      if (this.graphComponent.graph.nodeDefaults.style instanceof MyShapeNodeStyle) {
        realizertype = 1 // or 2
        style = new MyShapeNodeStyle()
        title = ''
      }
      if (this.graphComponent.graph.nodeDefaults.style instanceof TextNodeStyle) {
        realizertype = 1
        style = new TextNodeStyle()
      }
      if (this.graphComponent.graph.nodeDefaults.style instanceof MyImageNodeStyle) {
        realizertype = 0
        style = new MyImageNodeStyle(this.editorProps.base_url)
        labelLineColor = '#444444'
      }

      // get label position
      const labelPosition = NodeStyleSettings.getIndexOfPosition(ExteriorLabelModelPosition.SOUTH)

      const x = node.layout.x + (node.layout.width / 2)
      const y = node.layout.y + (node.layout.height / 2)
      const position = parseInt(x.toString()) + ',' + parseInt(y.toString())

      const font = NodeStyleSettings.getFontStringByNodeSetup(this.editorProps.node_setup)
      const textvisible = defaults && defaults.textvisible ? defaults.textvisible : '1'
      const object: MapObject = {
        mapobject_id: -1,
        position: position,
        width: defaults && defaults.width ? defaults.width : node.layout.width.toString(),
        height: defaults && defaults.height ? defaults.height : node.layout.height.toString(),
        title: title,
        realizertype: realizertype.toString(),
        shapetype: defaults && defaults.shapetype ? defaults.shapetype : 0,
        labelposition: labelPosition.toString(),
        textvisible: textvisible,
        textcolor: this.editorProps.node_setup.color,
        linecolor: '#444444',
        labellinecolor: labelLineColor,
        fillcolor: defaults && defaults.fillcolor ? defaults.fillcolor : this.editorProps.node_setup.background_color,
        font: font,
        is_title_custom: '1',
        imagepath: defaults && defaults.imagepath ? defaults.imagepath : null,
        image_file: defaults && defaults.image_file ? defaults.image_file : null,
        link: defaults && defaults.link ? defaults.link : null,
        isys_obj__id: defaults && defaults.isys_obj__id ? defaults.isys_obj__id : null
      }

      // assign map connection object to edge
      node.tag = object

      // set node style based on map object (to avoid using default node style)
      this.graphComponent.graph.setStyle(node, style)

      if (textvisible == '1') {
        this.graphComponent.graph.addLabel(node, title)
      }

      // save new node
      api('shd.map_object.create',
        {
          shd_map__id: this.apiProps.map_id,
          ...object
        },
        this.apiProps).then(res => {
        // save map connection ID
        object.mapobject_id = parseInt(res.result.response)
        node.tag = object
        this.updateStructureView()
      })
    }

    deleteSelection () {
      this.graphComponent.selection.selectedNodes.forEach((node) => {
        // delete connected edges
        node.ports.forEach(port => {
          const edges = this.graphComponent.graph.edgesAt(port)
          edges.forEach(edge => {
            const mapConnection: MapConnection = edge.tag as MapConnection
            api('shd.map_connection.delete', { mapconnection_id: mapConnection.mapconnection_id },
              this.apiProps).then(() => {})
          })
        })

        // delete node
        const mapObject: MapObject = node.tag as MapObject
        api('shd.map_object.delete', { mapobject_id: mapObject.mapobject_id },
          this.apiProps).then(() => {
          this.updateStructureView()
        })
      })
      super.deleteSelection()
    }
}
