import * as React from 'react'
import { ApiProps } from '../input_modes/NodeEditorInputMode'
import { api } from '../ajax'
import { ObjectType, TransferObject } from '../MapsCompose'

interface CmdbObjectMenuProps {
    api_props: ApiProps,
    base_url: string,
    object_types: ObjectType[],
    language: {[key: string]: string},
}

interface CmdbObjectMenuState {
    selected_object_type: number,
    objects: {[key: string]: CmdbObject},
    is_loading: boolean,
    filter: string,
    filtered_items: {[key: string]: CmdbObject},
}

interface CmdbObject {
    id: number,
    object_type: number,
    title: string,
    status: number,
    created?: string,
    changed?: string,
}

export class CmdbObjectMenu extends React.Component<CmdbObjectMenuProps, CmdbObjectMenuState> {
  constructor (props: CmdbObjectMenuProps) {
    super(props)
    this.handleObjectTypeChange = this.handleObjectTypeChange.bind(this)
    this.handleDragStart = this.handleDragStart.bind(this)
    this.handleFilterChange = this.handleFilterChange.bind(this)
    this.state = {
      selected_object_type: -1,
      objects: {},
      is_loading: false,
      filter: '',
      filtered_items: {}
    }
  }

  handleObjectTypeChange (evt: React.ChangeEvent<HTMLSelectElement>) {
    const object_type = parseInt(evt.currentTarget.value)
    this.setState((prevState) => ({ ...prevState, selected_object_type: object_type, filter: '', is_loading: true }))

    api('shd.query.read',
      {
        obj_type_id: object_type,
        columns: [
          { id: 'C__CATG__GLOBAL-title' },
          { id: 'C__CATG__GLOBAL-cmdb_status' },
          { id: 'C__CATG__GLOBAL-created' },
          { id: 'C__CATG__GLOBAL-changed' }
        ]
      },
      this.props.api_props).then(res => {
      const objects: {[key: string]: CmdbObject} = {}
      for (const key in res.result.content) {
        if (res.result.content.hasOwnProperty(key)) {
          const object = res.result.content[key]
          const id = parseInt(object.__id__)
          objects[id] = {
            id: id,
            title: object['C__CATG__GLOBAL-title'],
            status: object['C__CATG__GLOBAL-cmdb_status'],
            created: object['C__CATG__GLOBAL-created'],
            changed: object['C__CATG__GLOBAL-changed'],
            object_type: object_type
          }
        }
      }
      this.setState((prevState) => ({ ...prevState, objects: objects, filtered_items: objects, is_loading: false }))
    })
  }

  handleDragStart (evt: React.DragEvent<HTMLDivElement>) {
    const id = (evt.target as HTMLDivElement).id
    const object = this.state.objects[id]
    const objectType = this.props.object_types[object.object_type]
    const transfer: TransferObject = {
      transfer_type: 'cmdb-object',
      object_id: parseInt(id),
      object_title: object.title,
      icon: objectType.icon
    }
    evt.dataTransfer.setData('text/json', JSON.stringify(transfer))
  }

  handleFilterChange (evt: React.ChangeEvent<HTMLInputElement>) {
    const filter = evt.currentTarget.value
    let filteredItems: {[key: string]: CmdbObject} = {}
    if (filter != '') {
      for (const key in this.state.objects) {
        if (this.state.objects.hasOwnProperty(key)) {
          const object = this.state.objects[key]
          const title = object.title.toLowerCase()
          if (title.includes(filter.toLowerCase())) {
            filteredItems[key] = object
          }
        }
      }
    } else {
      filteredItems = this.state.objects
    }
    this.setState((prevState) => ({ ...prevState, filter: filter, filtered_items: filteredItems }))
  }

  render () {
    const types: ObjectType[] = []
    for (const key in this.props.object_types) {
      if (this.props.object_types.hasOwnProperty(key)) {
        types.push(this.props.object_types[key])
      }
    }

    types.sort((a: ObjectType, b: ObjectType) => {
      return ('' + a.title.toLowerCase()).localeCompare(b.title.toLowerCase())
    })

    const objectTypes = [<option key={-1} value={-1}>-</option>]
    types.forEach(objectType => {
      objectTypes.push(<option key={objectType.id} value={objectType.id}>{objectType.title}</option>)
    })

    const objects = []
    if (this.state.is_loading) {
      const spinnerUrl = this.props.base_url + '/images/ajax-loading.gif'
      objects.push(<tr key="loading">
        <td colSpan={5}><img src={spinnerUrl} alt="..." /></td>
      </tr>)
    } else {
      let count = 0

      // sort filtered items
      const items: CmdbObject[] = []
      for (const key in this.state.filtered_items) {
        if (this.state.filtered_items.hasOwnProperty(key)) {
          const object = this.state.filtered_items[key]
          items.push(object)
        }
      }
      items.sort((a: CmdbObject, b: CmdbObject) => {
        return ('' + a.title.toLowerCase()).localeCompare(b.title.toLowerCase())
      })

      items.forEach(object => {
        count++
        if (count > 10) {
          return
        }

        // const object = this.state.filtered_items[key]
        objects.push(<tr key={object.id} id={object.id.toString()} draggable={true} onDragStart={this.handleDragStart}>
          <td>
            <span className="icon"
              id={object.id.toString()}
              draggable={true}
              onDragStart={this.handleDragStart}
              title={this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__DRAG_HINT}>+</span>
          </td>
          <td>{object.title}</td>
          <td>{object.status}</td>
          <td>{object.created}</td>
          <td>{object.changed}</td>
        </tr>)

        if (count == 10) {
          objects.push(<tr key="more_objects">
            <td colSpan={5}>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__MORE_THAN_25_OBJECTS}</td>
          </tr>)
        }
      })

      if (Object.keys(this.state.objects).length == 0) {
        objects.push(<tr key="no_objects">
          <td colSpan={5}>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__OBJECT_TYPE_CONTAINS_NO_OBJECTS}</td>
        </tr>)
      }
    }

    return <div className="tools-menu cmdb-object-menu">
      <div>
        <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__OBJECT_TYPE}:</span>
        <select value={this.state.selected_object_type} onChange={this.handleObjectTypeChange} className="input input-small">
          {objectTypes}
        </select>
      </div>
      <div>
        <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__FILTER}:</span>
        <input type="text" value={this.state.filter} onChange={this.handleFilterChange} className="input input-small" />
      </div>
      <div>
        <table className="mainTable">
          <thead>
            <tr>
              <th>&nbsp;</th>
              <th>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__DESCRIPTION}</th>
              <th>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__STATUS}</th>
              <th>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__CREATED}</th>
              <th>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__CHANGED}</th>
            </tr>
          </thead>
          <tbody>
            {objects}
          </tbody>
        </table>
      </div>
    </div>
  }
}
