import * as React from 'react'
import { EdgeStyleSettings } from '../styles/EdgeStyleSettings'
import { EdgeSetupProps } from '../MapsCompose'

interface EdgeMenuProps {
    base_url: string,
    edge_setup: EdgeSetupProps,
    onEdgeSetupChange: Function,
    language: {[key: string]: string},
}

export class EdgeMenu extends React.Component<EdgeMenuProps, {}> {
    static readonly LINE_STROKES: {[key: string]: number} = {
      'shape-line-xs': EdgeStyleSettings.EDGE_STROKE_XS,
      'shape-line-s': EdgeStyleSettings.EDGE_STROKE_S,
      'shape-line-m': EdgeStyleSettings.EDGE_STROKE_M,
      'shape-line-l': EdgeStyleSettings.EDGE_STROKE_L,
      'shape-line-xl': EdgeStyleSettings.EDGE_STROKE_XL
    };

    static readonly LINE_TYPES: {[key: string]: number} = {
      'shape-line-s': EdgeStyleSettings.EDGE_STYLE_SOLID,
      'shape-line-dotted': EdgeStyleSettings.EDGE_STYLE_DOTTED,
      'shape-line-dashed': EdgeStyleSettings.EDGE_STYLE_DASHED
    };

    static readonly ARROWS: {[key: string]: number} = {
      'shape-arrow-none': EdgeStyleSettings.ARROW_TYPE_NONE,
      'shape-arrow-default': EdgeStyleSettings.ARROW_TYPE_DEFAULT,
      'shape-arrow-circle': EdgeStyleSettings.ARROW_TYPE_CIRCLE,
      'shape-arrow-diamond': EdgeStyleSettings.ARROW_TYPE_DIAMOND,
      'shape-arrow-short': EdgeStyleSettings.ARROW_TYPE_SHORT,
      'shape-arrow-triangle': EdgeStyleSettings.ARROW_TYPE_TRIANGLE
    };

    constructor (props: EdgeMenuProps) {
      super(props)
      this.handleStrokeChange = this.handleStrokeChange.bind(this)
      this.handleStyleChange = this.handleStyleChange.bind(this)
      this.handleSourceArrowChange = this.handleSourceArrowChange.bind(this)
      this.handleTargetArrowChange = this.handleTargetArrowChange.bind(this)
      this.handleLineColorChange = this.handleLineColorChange.bind(this)
    }

    handleStrokeChange (evt: React.ChangeEvent<HTMLInputElement>) {
      const edgeSetup = this.props.edge_setup
      edgeSetup.stroke = parseInt(evt.currentTarget.value)
      this.props.onEdgeSetupChange(edgeSetup, true)
    }

    handleStyleChange (evt: React.ChangeEvent<HTMLInputElement>) {
      const edgeSetup = this.props.edge_setup
      edgeSetup.style = parseInt(evt.currentTarget.value)
      this.props.onEdgeSetupChange(edgeSetup, true)
    }

    handleSourceArrowChange (evt: React.ChangeEvent<HTMLInputElement>) {
      const edgeSetup = this.props.edge_setup
      edgeSetup.source_arrow_type = parseInt(evt.currentTarget.value)
      this.props.onEdgeSetupChange(edgeSetup, true)
    }

    handleTargetArrowChange (evt: React.ChangeEvent<HTMLInputElement>) {
      const edgeSetup = this.props.edge_setup
      edgeSetup.target_arrow_type = parseInt(evt.currentTarget.value)
      this.props.onEdgeSetupChange(edgeSetup, true)
    }

    handleLineColorChange (evt: React.ChangeEvent<HTMLInputElement>) {
      const edgeSetup = this.props.edge_setup
      edgeSetup.line_color = evt.currentTarget.value
      this.props.onEdgeSetupChange(edgeSetup, true)
    }

    render () {
      const lineStrokes = []
      for (const className in EdgeMenu.LINE_STROKES) {
        if (EdgeMenu.LINE_STROKES.hasOwnProperty(className)) {
          const edge = EdgeMenu.LINE_STROKES[className]
          lineStrokes.push(
            <input type="radio" name="edge_stroke[]" key={'stroke_' + edge}
              checked={this.props.edge_setup.stroke == edge}
              onChange={this.handleStrokeChange}
              value={edge} />)
          lineStrokes.push(<span key={'shape_stroke_' + edge} className={'shape ' + className} />)
        }
      }

      const lineTypes = []
      for (const className in EdgeMenu.LINE_TYPES) {
        if (EdgeMenu.LINE_TYPES.hasOwnProperty(className)) {
          const edge = EdgeMenu.LINE_TYPES[className]
          lineTypes.push(
            <input type="radio" name="edge_style[]" key={'style_' + edge}
              checked={this.props.edge_setup.style == edge}
              onChange={this.handleStyleChange}
              value={edge} />)
          lineTypes.push(<span key={'shape_style_' + edge} className={'shape ' + className} />)
        }
      }

      const sourceArrows = []
      const targetArrows = []
      for (const className in EdgeMenu.ARROWS) {
        if (EdgeMenu.ARROWS.hasOwnProperty(className)) {
          const arrow = EdgeMenu.ARROWS[className]
          sourceArrows.push(
            <input type="radio" name="source_arrow[]" key={'source_' + arrow}
              checked={this.props.edge_setup.source_arrow_type == arrow}
              onChange={this.handleSourceArrowChange}
              value={arrow} />)
          sourceArrows.push(<span key={'shape_source_' + arrow} className={'shape ' + className} />)

          targetArrows.push(
            <input type="radio" name="target_arrow[]" key={'target_' + arrow}
              checked={this.props.edge_setup.target_arrow_type == arrow}
              onChange={this.handleTargetArrowChange}
              value={arrow} />)
          targetArrows.push(<span key={'shape_target_' + arrow} className={'shape rotate-180 ' + className} />)
        }
      }

      return <div className="tools-menu edge-menu">
        <div>
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__STROKE}:</span>
          {lineStrokes}
        </div>
        <div>
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__STYLE}:</span>
          {lineTypes}
        </div>
        <div>
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__SOURCE_ARROW}:</span>
          {sourceArrows}
        </div>
        <div>
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__TARGET_ARROW}:</span>
          {targetArrows}
        </div>
        <div className="color-picker">
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__LINE_COLOR}:</span>
          <input key="color" type="color" onChange={this.handleLineColorChange} value={this.props.edge_setup.line_color} />
        </div>
      </div>
    }
}
