import * as React from 'react'
import { ApiProps } from '../input_modes/NodeEditorInputMode'
import { api } from '../ajax'
import { TransferObject } from '../MapsCompose'

interface ShdImage {
  hash: string,
  title: string,
  path: string,
  data?: string,
  width?: number,
  height?: number,
}

interface ImageMenuProps {
    api_props: ApiProps,
    base_url: string,
    language: {[key: string]: string},
}

interface ImageMenuState {
    selected_path: string,
    paths: {[path: string]: string[]},
    images: {[path: string]: ShdImage[]},
    selected_image?: ShdImage,
    is_loading: boolean,
    is_image_loading: boolean,
}

export class ImageMenu extends React.Component<ImageMenuProps, ImageMenuState> {
  constructor (props: ImageMenuProps) {
    super(props)
    this.loadDirectory = this.loadDirectory.bind(this)
    this.handleLoadDirectory = this.handleLoadDirectory.bind(this)
    this.handleImagePreview = this.handleImagePreview.bind(this)
    this.handleDragStart = this.handleDragStart.bind(this)
    this.getPreview = this.getPreview.bind(this)
    this.state = {
      selected_path: '',
      paths: {},
      images: {},
      is_loading: false,
      is_image_loading: false
    }
  }

  componentDidMount () {
    this.loadDirectory('/')
  }

  loadDirectory (directory: string) {
    this.setState((prevState) => ({ ...prevState, selected_path: directory, is_loading: true }))
    const currentPath = directory
    api('shd.file.dir', { path: currentPath },
      this.props.api_props).then(res => {
      const images: ShdImage[] = []
      const paths = this.state.paths
      paths[currentPath] = []
      for (const key in res.result) {
        if (res.result.hasOwnProperty(key)) {
          const image = res.result[key]
          if (image.type == 'd') {
            paths[currentPath].push(image.name)
          }
          if (image.type == 'f') {
            images.push({
              hash: image.hash,
              path: image.path != '' ? image.path : '/',
              title: image.name
            })
          }
        }
      }

      const allImages = this.state.images
      allImages[currentPath] = images
      this.setState((prevState) => ({ ...prevState, paths: paths, images: allImages }))
    }).finally(() => {
      this.setState((prevState) => ({ ...prevState, is_loading: false }))
    })
  }

  handleLoadDirectory (evt: React.MouseEvent<HTMLSpanElement>) {
    const path = evt.currentTarget.id
    this.loadDirectory(path)
  }

  handleImagePreview (evt: React.MouseEvent<HTMLSpanElement>) {
    // get selected image
    const hash = evt.currentTarget.id
    let image: ShdImage = null
    for (const path in this.state.images) {
      if (this.state.images.hasOwnProperty(path)) {
        const images = this.state.images[path]
        images.forEach(img => {
          if (img.hash == hash) {
            image = img
          }
        })
      }
    }

    if (!image) {
      return
    }

    this.setState((prevState) => ({ ...prevState, selected_image: image, is_image_loading: true }))
    api('maps.image.read',
      {
        path: image.path + (image.path == '/' ? '' : '/') + image.title,
        include_file: 1
      },
      this.props.api_props).then(res => {
      const data = res.result.content
      const object = this.state.selected_image
      object.data = data
      object.width = res.result.width
      object.height = res.result.height
      this.setState((prevState) => ({ ...prevState, selected_image: object }))
    }).finally(() => {
      this.setState((prevState) => ({ ...prevState, is_image_loading: false }))
    })
  }

  handleDragStart (evt: React.DragEvent<HTMLElement>) {
    const object = this.state.selected_image
    const path = object.path + (object.path != '/' ? '/' : '') + object.title
    const transfer: TransferObject = {
      transfer_type: 'image',
      object_id: 0,
      object_title: path,
      icon: this.state.selected_image.data,
      image_width: this.state.selected_image.width,
      image_height: this.state.selected_image.height
    }
    evt.dataTransfer.setData('text/json', JSON.stringify(transfer))
  }

  getPreview () {
    if (!(this.state.selected_image && this.state.selected_image.data) && !this.state.is_image_loading) {
      return null
    }

    const spinnerUrl = this.props.base_url + '/images/ajax-loading.gif'
    let img = null
    let info = null
    if (this.state.selected_image && this.state.selected_image.data) {
      img = <img src={'data:image/png;base64, ' + this.state.selected_image.data}
        alt={this.state.selected_image.title}
        title={this.state.selected_image.title}
        id={this.state.selected_image.title}
        draggable={true}
        onDragStart={this.handleDragStart} />
      info = <span>
        {this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__IMAGE_SIZE}:&nbsp;
        {this.state.selected_image.width + ' x ' + this.state.selected_image.height + ' px'}
      </span>
    }
    if (this.state.is_image_loading) {
      img = <img src={spinnerUrl}
        alt="..." />
    }
    return <ol key="preview" className="preview">
      <li>
        {img}
        {info}
      </li>
    </ol>
  }

  render () {
    const ols = []
    if (this.state.is_loading) {
      const spinnerUrl = this.props.base_url + '/images/ajax-loading.gif'
      ols.push(<ol key="loading"><li key="loading">
        <img src={spinnerUrl} alt="..." />
      </li></ol>)
    } else {
      const sp = this.state.selected_path.substr(1)
      const selectedPath = sp.split('/')
      selectedPath.unshift('/') // add root directory

      let currentPath = ''
      selectedPath.forEach(dir => {
        if (dir == '') {
          return
        }
        currentPath += dir

        const paths: string[] = this.state.paths[currentPath] ? this.state.paths[currentPath] : []
        const images: ShdImage[] = this.state.images[currentPath] ? this.state.images[currentPath] : []
        currentPath += dir == '/' ? '' : '/' // add trailing slash

        const dirItems: JSX.Element[] = []
        paths.forEach(path => {
          const completePath = currentPath + path
          let className = 'dir '
          className += this.state.selected_path.startsWith(completePath) ? 'selected' : ''
          dirItems.push(<li key={completePath}>
            <button id={completePath} type="button" className={className} onClick={this.handleLoadDirectory}>
              {path}
            </button>
          </li>)
        })

        images.forEach(image => {
          let className = 'image '
          className += this.state.selected_image && image.hash == this.state.selected_image.hash ? 'selected' : ''
          dirItems.push(<li key={image.hash} id={image.hash}>
            <button id={image.hash} type="button" className={className} onClick={this.handleImagePreview}>
              {image.title}
            </button>
          </li>)
        })
        ols.push(<ol key={currentPath}>
          {dirItems}
        </ol>)
      })
    }

    return <div className="tools-menu image-menu">
      <div>
        {ols}
        {this.getPreview()}
      </div>
    </div>
  }
}
