'use strict'

import 'yfiles/view-layout-bridge.js'
import {
  Arrow,
  ArrowType,
  DashStyle,
  PolylineEdgeStyle,
  Stroke
} from 'yfiles/view-component'
import { ArcEdgeStyle } from 'yfiles/styles-other'
import { EdgeSetupProps, MapConnection } from '../MapsCompose'
import {IEdgeStyle} from "yfiles";

export class EdgeStyleSettings {
    static readonly DEFAULT_LINE_COLOR = '000000';

    static readonly ARROW_TYPE_NONE = 1;
    static readonly ARROW_TYPE_DEFAULT = 2;
    static readonly ARROW_TYPE_TRIANGLE = 3;
    static readonly ARROW_TYPE_DIAMOND = 5;
    static readonly ARROW_TYPE_SHORT = 7;
    static readonly ARROW_TYPE_CIRCLE = 11;
    static readonly arrow_types: {[key: number]: ArrowType} = {
      1: ArrowType.NONE,
      2: ArrowType.DEFAULT,
      3: ArrowType.TRIANGLE,
      5: ArrowType.DIAMOND,
      7: ArrowType.SHORT,
      8: ArrowType.SIMPLE,
      11: ArrowType.CIRCLE
    };

    static readonly EDGE_STROKE_XS = 1;
    static readonly EDGE_STROKE_S = 2;
    static readonly EDGE_STROKE_M = 3;
    static readonly EDGE_STROKE_L = 4;
    static readonly EDGE_STROKE_XL = 5;
    static readonly arrow_scale: {[key: number]: number} = {
      1: 1,
      2: 1.2,
      3: 1.4,
      4: 1.6,
      5: 1.8
    };

    static readonly EDGE_STYLE_SOLID = 0;
    static readonly EDGE_STYLE_DASHED = 1;
    static readonly EDGE_STYLE_DOTTED = 2;
    static readonly line_style: {[key: number]: DashStyle} = {
      0: DashStyle.SOLID,
      1: DashStyle.DASH,
      2: DashStyle.DASH_DOT_DOT
    };

    static getEdgeStyle (edgeData: MapConnection): IEdgeStyle {
      const sourceArrow = EdgeStyleSettings.getEdgeArrow(parseInt(edgeData.sourcearrowtype), parseInt(edgeData.linewidth), edgeData.linecolor)
      const targetArrow = EdgeStyleSettings.getEdgeArrow(parseInt(edgeData.targetarrowtype), parseInt(edgeData.linewidth), edgeData.linecolor)
      const stroke = EdgeStyleSettings.getEdgeStroke(parseInt(edgeData.linewidth), parseInt(edgeData.linestyle), edgeData.linecolor)
      if (edgeData.realizertype == '1') {
        const style = new ArcEdgeStyle()
        style.fixedHeight = true
        style.height = 0.25
        style.sourceArrow = sourceArrow
        style.targetArrow = targetArrow
        style.stroke = stroke
        return style
      } else {
        const style = new PolylineEdgeStyle()
        style.sourceArrow = sourceArrow
        style.targetArrow = targetArrow
        style.stroke = stroke
        return style
      }
    }

    static getEdgeStroke (lineWidth?: number, lineStyle?: number, lineColor?: string) {
      lineWidth = lineWidth || EdgeStyleSettings.EDGE_STROKE_S
      lineStyle = lineStyle || EdgeStyleSettings.EDGE_STYLE_SOLID
      lineColor = lineColor || EdgeStyleSettings.DEFAULT_LINE_COLOR
      const color = lineColor
      const stroke = new Stroke(color, lineWidth)
      stroke.dashStyle = EdgeStyleSettings.getLineStyle(lineStyle)
      return stroke
    }

    static getEdgeArrow (arrowType: number, lineWidth?: number, lineColor?: string) {
      lineWidth = lineWidth || EdgeStyleSettings.EDGE_STROKE_S
      lineColor = lineColor || EdgeStyleSettings.DEFAULT_LINE_COLOR
      const color = lineColor
      return new Arrow({
        color: color,
        scale: EdgeStyleSettings.getArrowScale(lineWidth),
        type: EdgeStyleSettings.getArrow(arrowType)
      })
    }

    static getArrow (type: number) {
      return EdgeStyleSettings.arrow_types[type] ? EdgeStyleSettings.arrow_types[type] : ArrowType.DEFAULT
    }

    static getArrowScale (width: number) {
      return EdgeStyleSettings.arrow_scale[width] ? EdgeStyleSettings.arrow_scale[width] : 1.4
    }

    static getLineStyle (style: number) {
      return EdgeStyleSettings.line_style[style] ? EdgeStyleSettings.line_style[style] : DashStyle.SOLID
    }

    static getEdgeSetupByMapConnection (connection: MapConnection) {
      const edgeSetup: EdgeSetupProps = {
        stroke: parseInt(connection.linewidth),
        style: parseInt(connection.linestyle),
        line_color: connection.linecolor,
        source_arrow_type: parseInt(connection.sourcearrowtype),
        target_arrow_type: parseInt(connection.targetarrowtype)
      }
      return edgeSetup
    }
}
