import {
    Font,
    ILabel,
    IOrientedRectangle,
    IRenderContext,
    LabelStyleBase,
    Size,
    SvgVisual,
    YObject
} from 'yfiles'
import {NodeStyleSettings} from "./NodeStyleSettings";
import {MapObject, NodeSetupProps} from "../MapsCompose";

type Cache = {
    layout: IOrientedRectangle;
    text: string;
    font: Font
}

export default class HtmlLabelStyle extends LabelStyleBase {

    protected font: Font;

    constructor(nodeSetup?: NodeSetupProps) {
        super()
        this.font = new Font({
            fontFamily: nodeSetup ? nodeSetup.family : 'Dialog',
            fontSize: nodeSetup ? nodeSetup.size : 14
        })
    }

    createVisual(context: IRenderContext, label: ILabel): SvgVisual {
        const labelLayout = label.layout

        if (label.owner.tag) {
            const data = label.owner.tag as MapObject
            this.updateFont(data)
        }

        const foreignObject = document.createElementNS('http://www.w3.org/2000/svg', 'foreignObject')
        foreignObject.setAttribute('x', '0')
        foreignObject.setAttribute('y', '0')

        const div = document.createElement('div')
        div.style.setProperty('overflow', 'hidden')
        foreignObject.setAttribute('width', `${labelLayout.width}`)
        foreignObject.setAttribute('height', `${labelLayout.height}`)
        div.style.setProperty('width', `${labelLayout.width}px`)
        div.style.setProperty('height', `${labelLayout.height}px`)
        div.style.setProperty('font-family', this.font.fontFamily)
        div.style.setProperty('font-size', `${this.font.fontSize}px`)
        div.style.setProperty('font-weight', `${this.font.fontWeight}`)
        div.style.setProperty('font-style', `${this.font.fontStyle}`)

        if (label.owner.tag) {
            const data = label.owner.tag as MapObject
            div.style.setProperty('color', data.textcolor)
            div.style.setProperty('background-color', data.fillcolor)
        }

        div.innerHTML = label.text
        foreignObject.appendChild(div)

        // move container to correct location
        const transform = LabelStyleBase.createLayoutTransform(context, label.layout, true)
        transform.applyTo(foreignObject)

        // Get the necessary data for rendering of the label and store information with the visual
        ;(foreignObject as any)['data-cache'] = createRenderDataCache(label, this.font)

        return new SvgVisual(foreignObject)
    }

    updateVisual(context: IRenderContext, oldVisual: SvgVisual, label: ILabel): SvgVisual {
        const element = oldVisual.svgElement
        if (element === null || element.childElementCount !== 1) {
            // re-create from scratch if this is not the case
            return this.createVisual(context, label)
        }

        // get the data with which the old visual was created
        const oldCache = (element as any)['data-cache'] as Cache

        if (label.owner.tag) {
            const data = label.owner.tag
            this.updateFont(data)
        }

        // get the data for the new visual
        const newCache = createRenderDataCache(label, this.font)

        // update elements if they have changed
        const foreignObject = element
        const div = foreignObject.firstElementChild as HTMLDivElement
        if (!YObject.equals(oldCache.layout, newCache.layout)) {
            const labelLayout = label.layout
            foreignObject.setAttribute('width', `${labelLayout.width}`)
            foreignObject.setAttribute('height', `${labelLayout.height}`)

            div.style.setProperty('width', `${labelLayout.width}px`)
            div.style.setProperty('height', `${labelLayout.height}px`)
        }
        if (!oldCache.font.equals(newCache.font)) {
            div.style.setProperty('font-family', this.font.fontFamily)
            div.style.setProperty('font-size', `${this.font.fontSize}px`)
            div.style.setProperty('font-weight', `${this.font.fontWeight}`)
            div.style.setProperty('font-style', `${this.font.fontStyle}`)
        }

        if (oldCache.text !== newCache.text) {
            div.innerHTML = label.text
        }

        // update the cache
        ;(element as any)['data-cache'] = newCache

        // move container to correct location
        const transform = LabelStyleBase.createLayoutTransform(context, label.layout, true)
        transform.applyTo(foreignObject)

        return oldVisual
    }

    getPreferredSize(label: ILabel): Size {
        const div = document.createElement('div')
        div.style.setProperty('display', 'inline-block')
        div.innerHTML = label.text
        document.body.appendChild(div)
        const clientRect = div.getBoundingClientRect()
        document.body.removeChild(div)
        const scale = this.font.fontSize / 11
        return new Size(clientRect.width * scale, clientRect.height * scale)
    }

    updateFont (data: MapObject) {
        const font = NodeStyleSettings.getFontByFontString(data.font)
        if (font) {
            this.font = font
        }
    }
}

function createRenderDataCache(label: ILabel, font: Font): Cache {
    return {
        text: label.text,
        font,
        layout: label.layout
    }
}
