'use strict'

import 'yfiles/view-layout-bridge.js'
import { ExteriorLabelModelPosition, Font, FontStyle } from 'yfiles/view-component'
import { NodeSetupProps, MapObject } from '../MapsCompose'
import { TextNodeStyle } from './TextNodeStyle'
import { MyShapeNodeStyle } from './ShapeNodeStyle'
import { MyImageNodeStyle } from './ImageNodeStyle'
import { Point } from 'yfiles'

export class NodeStyleSettings {
    static readonly positions: {[key: number]: ExteriorLabelModelPosition} = {
      104: ExteriorLabelModelPosition.NORTH_WEST,
      105: ExteriorLabelModelPosition.NORTH_EAST,
      106: ExteriorLabelModelPosition.SOUTH_EAST,
      107: ExteriorLabelModelPosition.SOUTH_WEST,
      108: ExteriorLabelModelPosition.NORTH,
      109: ExteriorLabelModelPosition.SOUTH,
      110: ExteriorLabelModelPosition.EAST,
      111: ExteriorLabelModelPosition.WEST
    };

    static getNodeStyle (nodeData: MapObject, baseUrl: string) {
      if (nodeData.realizertype == '2' || nodeData.realizertype == '1') {
        // shape node or text node
        if (parseInt(nodeData.shapetype) == 0 && parseInt(nodeData.textalignment) == 0) {
          return new TextNodeStyle()
        } else {
          return new MyShapeNodeStyle()
        }
      }
      if (nodeData.realizertype == '0') {
        // image node
        return new MyImageNodeStyle(baseUrl)
      }
    }

    static getPosition (position: number) {
      return NodeStyleSettings.positions[position] ? NodeStyleSettings.positions[position] : ExteriorLabelModelPosition.SOUTH
    }

    static getIndexOfPosition (position: ExteriorLabelModelPosition) {
      let index = 109
      for (const key in NodeStyleSettings.positions) {
        if (NodeStyleSettings.positions.hasOwnProperty(key)) {
          const pos = NodeStyleSettings.positions[key]
          if (pos == position) {
            index = parseInt(key)
          }
        }
      }
      return index
    }

    static getNodeSetupByMapObject (mapObject: MapObject) {
      const fnt = mapObject.font.split(';')
      if (fnt.length != 3) {
        return
      }

      const style = parseInt(fnt[1])
      const nodeSetup: NodeSetupProps = {
        family: fnt[0],
        size: parseInt(fnt[2]),
        color: mapObject.textcolor,
        background_color: mapObject.fillcolor,
        is_bold: style == 1 || style == 3,
        is_italic: style == 2 || style == 3,
        shape_type: parseInt(mapObject.shapetype)
      }
      return nodeSetup
    }

    static getFontByFontString (font: string) {
      const fnt = font.split(';')
      if (fnt.length != 3) {
        return
      }

      const style = parseInt(fnt[1])
      return new Font({
        fontFamily: fnt[0],
        fontSize: parseInt(fnt[2]),
        fontWeight: style == 1 || style == 3 ? 'bold' : 'normal',
        fontStyle: style == 2 || style == 3 ? FontStyle.ITALIC : FontStyle.NORMAL
      })
    }

    static getFontStringByNodeSetup (nodeSetup: NodeSetupProps) {
      let font = nodeSetup.family ? nodeSetup.family : 'Dialog'
      let fontStyle = 0
      if (nodeSetup.is_bold) {
        fontStyle = 1
      }
      if (nodeSetup.is_italic) {
        fontStyle = 2
      }
      if (nodeSetup.is_bold && nodeSetup.is_italic) {
        fontStyle = 3
      }

      font += ';' + fontStyle + ';'
      font += nodeSetup.size ? nodeSetup.size : 14
      return font
    }

    static getLabelPosition (nodeCenter: Point, labelCenter: Point) {
      // todo: find a better way to get label position
      let pos = ExteriorLabelModelPosition.SOUTH
      if (nodeCenter.x == labelCenter.x && nodeCenter.y > labelCenter.y) {
        pos = ExteriorLabelModelPosition.NORTH
      }
      if (nodeCenter.x == labelCenter.x && nodeCenter.y < labelCenter.y) {
        pos = ExteriorLabelModelPosition.SOUTH
      }
      if (nodeCenter.y == labelCenter.y && nodeCenter.x > labelCenter.x) {
        pos = ExteriorLabelModelPosition.WEST
      }
      if (nodeCenter.y == labelCenter.y && nodeCenter.x < labelCenter.x) {
        pos = ExteriorLabelModelPosition.EAST
      }
      if (nodeCenter.x > labelCenter.x && nodeCenter.y > labelCenter.y) {
        pos = ExteriorLabelModelPosition.NORTH_WEST
      }
      if (nodeCenter.x > labelCenter.x && nodeCenter.y < labelCenter.y) {
        pos = ExteriorLabelModelPosition.SOUTH_WEST
      }
      if (nodeCenter.x < labelCenter.x && nodeCenter.y > labelCenter.y) {
        pos = ExteriorLabelModelPosition.NORTH_EAST
      }
      if (nodeCenter.x < labelCenter.x && nodeCenter.y < labelCenter.y) {
        pos = ExteriorLabelModelPosition.SOUTH_EAST
      }
      return pos
    }
}
