'use strict'

import 'yfiles/view-layout-bridge.js'
import { INode, IRenderContext, SvgVisual } from 'yfiles/view-component'
import { AbstractNodeStyle } from './AbstractNodeStyle'

export class MyShapeNodeStyle extends AbstractNodeStyle {
  createVisual (context: IRenderContext, node: INode) {
    // This implementation creates a 'g' element and uses it as a container for the rendering of the node.
    const g = window.document.createElementNS('http://www.w3.org/2000/svg', 'g')
    // Render the node
    this.render(context, node, g)
    // set the location
    SvgVisual.setTranslate(g, node.layout.x, node.layout.y)
    return new SvgVisual(g)
  }

  render (context: IRenderContext, node: INode, container: Element) {
    // Create Defs section in container
    const defs = window.document.createElementNS('http://www.w3.org/2000/svg', 'defs')
    container.appendChild(defs)

    let shapeType
    if (node.tag && node.tag.shapetype) {
      shapeType = node.tag.shapetype
    }

    let shape
    shapeType = parseInt(shapeType)
    // with realizer = 1
    switch (shapeType) {
      case 0: shape = this.getRect(node); break // Rechteck
      case 1: shape = this.getRoundRect(node); break // Rechteck, runde Ecken
      case 2: shape = MyShapeNodeStyle.getEllipse(node); break // Kreis
      case 3: shape = this.getRhomboid(node); break // Parallologramm
      case 4: shape = this.getBanner(node); break // Banner
      case 5: shape = this.getTriangle(node); break // Dreieck
      case 7: shape = this.getStopSign(node); break // Stoppschild
      case 8: shape = this.getRhomb(node); break // Raute
      case 9: shape = this.getConeTop(node); break // Trichter oben
      case 10: shape = this.getConeBottom(node); break // Trichter unten
      default: shape = this.getRect(node); break
    }

    // set fill color
    const color = this.getNodeColor(node)
    shape.setAttribute('fill', color)

    // set border color
    shape.setAttribute('stroke-width', '0.8')
    shape.setAttribute('stroke', this.getBorderColor(node))

    if (shape) {
      container.appendChild(shape)
    }
  }

  private getRoundRect (node: INode) {
    const rect = super.getRect(node)
    rect.setAttribute('rx', '10')
    rect.setAttribute('ry', '10')
    return rect
  }

  private static getEllipse (node: INode) {
    const shape = window.document.createElementNS('http://www.w3.org/2000/svg', 'ellipse')
    const nodeSize = node.layout.toSize()
    const w = nodeSize.width * 0.5
    const h = nodeSize.height * 0.5
    shape.cx.baseVal.value = w
    shape.cy.baseVal.value = h
    shape.rx.baseVal.value = w
    shape.ry.baseVal.value = h
    return shape
  }

  private getTriangle (node: INode) {
    const nodeSize = node.layout.toSize()
    const w2 = nodeSize.width * 0.5
    const coords = [
      [0, nodeSize.height],
      [nodeSize.width, nodeSize.height],
      [w2, 0],
      [0, nodeSize.height]
    ]
    return this.coordsToShape(coords)
  }

  private getRhomb (node: INode) {
    const nodeSize = node.layout.toSize()
    const w2 = nodeSize.width * 0.5
    const h2 = nodeSize.height * 0.5
    const coords = [
      [0, h2],
      [w2, nodeSize.height],
      [nodeSize.width, h2],
      [w2, 0],
      [0, h2]
    ]
    return this.coordsToShape(coords)
  }

  private getRhomboid (node: INode) {
    const nodeSize = node.layout.toSize()
    const offset = 14
    const coords = [
      [0, nodeSize.height],
      [nodeSize.width - offset, nodeSize.height],
      [nodeSize.width, 0],
      [offset, 0],
      [0, nodeSize.height]
    ]
    return this.coordsToShape(coords)
  }

  private getBanner (node: INode) {
    const nodeSize = node.layout.toSize()
    const h2 = nodeSize.height * 0.5
    const offset = 14
    const coords = [
      [0, h2],
      [offset, nodeSize.height],
      [nodeSize.width - offset, nodeSize.height],
      [nodeSize.width, h2],
      [nodeSize.width - offset, 0],
      [offset, 0],
      [0, h2]
    ]
    return this.coordsToShape(coords)
  }

  private getStopSign (node: INode) {
    const nodeSize = node.layout.toSize()
    const w3 = nodeSize.width * 0.33
    const h3 = nodeSize.height * 0.33
    const coords = [
      [0, h3],
      [0, h3 * 2],
      [w3, nodeSize.height],
      [w3 * 2, nodeSize.height],
      [nodeSize.width, h3 * 2],
      [nodeSize.width, h3],
      [w3 * 2, 0],
      [w3, 0],
      [0, h3]
    ]
    return this.coordsToShape(coords)
  }

  private getConeTop (node: INode) {
    const nodeSize = node.layout.toSize()
    const offset = 30
    const coords = [
      [0, nodeSize.height],
      [nodeSize.width, nodeSize.height],
      [nodeSize.width - offset, 0],
      [offset, 0],
      [0, nodeSize.height]
    ]
    return this.coordsToShape(coords)
  }

  private getConeBottom (node: INode) {
    const nodeSize = node.layout.toSize()
    const offset = 30
    const coords = [
      [0, 0],
      [offset, nodeSize.height],
      [nodeSize.width - offset, nodeSize.height],
      [nodeSize.width, 0],
      [0, 0]
    ]
    return this.coordsToShape(coords)
  }

  private coordsToShape (coords: number[][]) {
    const shape = window.document.createElementNS('http://www.w3.org/2000/svg', 'polyline')
    let points = ''
    coords.forEach(c => {
      if (points.length > 0) {
        points += ' '
      }
      points += c[0] + ', ' + c[1]
    })
    shape.setAttribute('points', points)
    return shape
  }

  getDefaultFillColor () {
    return '#7784AA'
  }

  getDefaultBorderColor () {
    return '#444444'
  }
}
