'use strict'

import 'yfiles/view-layout-bridge.js'
import {
  Font,
  ILabel,
  IOrientedRectangle,
  IRenderContext,
  LabelStyleBase,
  Size,
  SvgVisual,
  TextRenderSupport,
  TextWrapping
} from 'yfiles/view-component'
import { NodeSetupProps, MapObject } from '../MapsCompose'
import { NodeStyleSettings } from './NodeStyleSettings'
import { INode } from 'yfiles'

export class TextLabelStyle extends LabelStyleBase {
    protected font: Font;

    constructor (nodeSetup?: NodeSetupProps) {
      super()
      this.font = new Font({
        fontFamily: nodeSetup ? nodeSetup.family : 'Dialog',
        fontSize: nodeSetup ? nodeSetup.size : 14
      })
    }

    render (label: ILabel, container: SVGGElement, labelLayout: IOrientedRectangle) {
      const rect: SVGRectElement = this.createBorderRect(label, container, labelLayout)
      let text: SVGTextElement
      if (container.childElementCount > 1) {
        text = container.childNodes.item(1) as SVGTextElement
      } else {
        text = window.document.createElementNS('http://www.w3.org/2000/svg', 'text')
        container.appendChild(text)
      }

      if (label.owner.tag) {
        const data = label.owner.tag
        this.updateFont(data)
        text.setAttribute('fill', data.textcolor)
      }

      // assign all the values of the font to the text element's attributes
      this.font.applyTo(text)

      // SVG does not provide out-of-the box text wrapping.
      // The following line uses a convenience method that implements text wrapping
      // with ellipsis by splitting the text and inserting tspan elements as children
      // of the text element. It is not mandatory to use this method, since the same
      // things could be done manually.
      const textContent = TextRenderSupport.addText(text, label.text, this.font, labelLayout.toSize(), TextWrapping.NONE)

      // calculate the size of the text element
      const textSize = TextRenderSupport.measureText(textContent, this.font)

      // calculate horizontal offset for centered alignment
      let translateX = (labelLayout.width - textSize.width) * 0.5

      // calculate vertical offset for centered alignment
      let translateY = (labelLayout.height - textSize.height) * 0.5

      text.setAttribute('transform', `translate(${translateX} ${translateY})`)

      // resize and place border rectangle
      const offsetWidth = 10
      const offsetHeight = 4
      let width = textSize.width + offsetWidth
      const height = textSize.height + offsetHeight
      rect.setAttribute('width', width.toString())
      rect.setAttribute('height', height.toString())

      translateX -= (offsetWidth / 2)
      translateY -= (offsetHeight / offsetHeight)
      rect.setAttribute('transform', `translate(${translateX} ${translateY})`)

      if (label.owner instanceof INode) {
        const node = label.owner as INode
        if (node.tag && node.tag.realizertype == '0' && width < node.layout.width) {
          // resize label to node's width
          const diff = node.layout.width - width
          width += diff
          rect.setAttribute('width', width.toString())
          translateX -= (diff / 2)
          rect.setAttribute('transform', `translate(${translateX} ${translateY})`)
        }
      }

      while (container.childElementCount > 2) {
        container.removeChild(container.childNodes.item(2))
      }
    }

    createBorderRect (label: ILabel, container: SVGGElement, labelLayout: IOrientedRectangle) {
      let rect: SVGRectElement
      if (container.childElementCount > 0) {
        rect = container.childNodes.item(0) as SVGRectElement
      } else {
        rect = window.document.createElementNS('http://www.w3.org/2000/svg', 'rect')
        container.appendChild(rect)
      }
      rect.width.baseVal.value = labelLayout.width
      rect.height.baseVal.value = labelLayout.height

      // border
      const mapObject: MapObject = label.owner.tag ? label.owner.tag as MapObject : null
      let borderColor = 'none'
      if (mapObject && mapObject.labellinecolor) {
        borderColor = mapObject.labellinecolor
      }
      rect.setAttribute('stroke', borderColor)
      rect.setAttribute('stroke-width', '0.5')

      // background color
      let color = null
      if (mapObject && mapObject.realizertype == '0' && mapObject.fillcolor != null) {
        // use always fillcolor on image nodes
        color = mapObject.fillcolor;
      }

      if (mapObject && mapObject.status != null) {
        const status = mapObject.status
        switch (status) {
          case -1: color = '#888888'; break
          case 0: color = '#78D168'; break
          case 1: color = '#FCFA90'; break
          case 2: color = '#E7A1A2'; break
          case 3: color = '#FAB47A'; break
          case 4: color = '#E7A1A2'; break
          case 5: color = '#FAB47A'; break
        }
        this.updateFont(mapObject)
      }

      const htmlColor = color || 'none'
      rect.setAttribute('fill', htmlColor)
      return rect
    }

    createVisual (context: IRenderContext, label: ILabel) {
      // This implementation creates a 'g' element and uses it for the rendering of the label.
      const g: SVGGElement = window.document.createElementNS('http://www.w3.org/2000/svg', 'g')
      // Render the label
      this.render(label, g, label.layout)
      // move container to correct location
      const transform = LabelStyleBase.createLayoutTransform(label.layout, true)
      transform.applyTo(g)
      // set data item
      g.setAttribute('data-internalId', 'MySimpleLabel')
      // todo: replace next line
      // g.setAttribute('data-item', label);
      return new SvgVisual(g)
    }

    getPreferredSize (label: ILabel) {
      // calculate width depending on text length
      let w = label.text.length * 7
      if (label.text.length < 10) {
        // add an offset for short labels
        w += 15
      }

      // add more space for upper letters
      if (label.text.length > 0) {
        const upperLetters = label.text.replace(/[^A-Z]/g, '').length
        w += upperLetters * 2
      }

      return new Size(w, 18)
    }

    updateFont (data: MapObject) {
      const font = NodeStyleSettings.getFontByFontString(data.font)
      if (font) {
        this.font = font
      }
    }
}
